from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "MarginTradingStockMapping",
    "MarginTradingStock",
]


class MarginTradingStockMapping(BaseMapping):
    """字段映射 融资融券交易明细-个股融资融券"""
    columns: Dict = {
        "DATE": "日期",
        "MARKET": "市场",
        "SCODE": "证券代码",
        "SECNAME": "证券名称",
        "RZYE": "融资余额",
        "RQYL": "融券余额",
        "RZRQYE": "融资融券余额",
        "RQYE": "融券余量",
        "RQMCL": "融券卖出量",
        "RZRQYECZ": "融资融券余额调整",
        "RZMRE": "融资买入额",
        "SZ": "市值",
        "RZYEZB": "融资余额占比",
        "RZMRE3D": "3日融资买入额",
        "RZMRE5D": "5日融资买入额",
        "RZMRE10D": "10日融资买入额",
        "RZCHE": "融资偿还额",
        "RZCHE3D": "3日融资偿还额",
        "RZCHE5D": "5日融资偿还额",
        "RZCHE10D": "10日融资偿还额",
        "RZJME": "融资交易额",
        "RZJME3D": "3日融资交易额",
        "RZJME5D": "5日融资交易额",
        "RZJME10D": "10日融资交易额",
        "RQMCL3D": "3日融券卖出量",
        "RQMCL5D": "5日融券卖出量",
        "RQMCL10D": "10日融券卖出量",
        "RQCHL": "融券持有量",
        "RQCHL3D": "3日融券持有量",
        "RQCHL5D": "5日融券持有量",
        "RQCHL10D": "10日融券持有量",
        "RQJMG": "融券交易量",
        "RQJMG3D": "3日融券交易量",
        "RQJMG5D": "5日融券交易量",
        "RQJMG10D": "10日融券交易量",
        "SPJ": "收盘价",
        "ZDF": "涨跌幅",
        "RCHANGE3DCP": "3日变动幅度",
        "RCHANGE5DCP": "5日变动幅度",
        "RCHANGE10DCP": "10日变动幅度",
        "KCB": "可转债",
        "TRADE_MARKET_CODE": "交易市场代码",
        "TRADE_MARKET": "交易市场",
        "FIN_BALANCE_GR": "融资余额增长率",
        "SECUCODE": "证券代码"
    }


class MarginTradingStock(APIDataV1RequestData):
    """查询 融资融券交易明细-个股融资融券"""
    def __init__(
            self,
            market: Literal["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"] = None,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            cycle: Literal[1, 3, 5, 10] = 1,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.margin import *

            # 查询 融资融券交易明细-个股融资融券
            data = MarginTradingStock(date="2024-10-30", cycle=1).load()
            data.to_frame(chinese_column=True)

            data = MarginTradingStock(market="沪深A股", date="2024-10-30", cycle=1).load()
            data.to_frame(chinese_column=True)

            data = MarginTradingStock(market="科创板", date="2024-10-30", cycle=1).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场名称: `["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]`
            start_date: 开始日期
            end_date: 结束日期
            date: 日期
            cycle: 统计周期
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.market = market
        self.cycle = cycle
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.size = size
        self.mapping = MarginTradingStockMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="融资融券交易明细-个股融资融券")
        self.conditions = []
        self.validate_date_format(date=[date, start_date, end_date])

    def params_filter(self) -> str:
        """"""
        market_mapping = {
            "沪深A股": '"069001001001","069001001006","069001002001","069001002002","069001002003"',
            "沪市A股": '"069001001001","069001001006"',
            "科创板": '"069001001006"',
            "深市A股": '"069001002001","069001002002","069001002003"',
            "创业板": '"069001002002"',
            "京市A股": '"069001017"',
        }
        self.filter_report_date(date_name="DATE")
        if self.market:
            self.conditions.append(f'(TRADE_MARKET_CODE in ({market_mapping[self.market]}))')
        return "".join(self.conditions)

    def params_sort_column(self):
        """"""
        if self.cycle == 1:
            return "RZJME"
        else:
            return f"RZJME{self.cycle}D"

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": "RPTA_WEB_RZRQ_GGMX",
            "sortColumns": self.params_sort_column(),
            "sortTypes": "-1",
            "pageSize": self.size,
            "filter": self.params_filter(),
            "pageNo": "1",
        }
        return self.base_param(update=params)



from .asset_invest import *
from .boom_index import *
from .cpi import *
from .currency_supply import *
from .customs import *
from .deposit_rate import *
from .deposit_reserve import *
from .faith_index import *
from .fdi import *
from .forex_deposit import *
from .forex_loan import *
from .gdp import *
from .gold_currency import *
from .goodes_index import *
from .gov_income import *
from .hose_index import *
from .imp_intrestraten import *
from .indus_grow import *
from .lpr import *
from .new_loan import *
from .oil_price import *
from .pmi import *
from .ppi import *
from .stock_open import *
from .stock_statistics import *
from .tax import *
from .total_retail import *
from .transfer_fund import *

from .index import *

# https://data.eastmoney.com/center/macro.html



import unittest
from wisecon.fund import *


class TestFundTrend(unittest.TestCase):
    def test_fund_trend(self):
        for period in ["1m", "3m", "6m", "1y", "3y", "5y", "this_year", "all"]:
            data = FundTrend(fund_code="000001", period=period).load()
            # data.pprint_metadata()
            print(f"{period} - {data.to_frame().shape}")
            print(data.to_frame(chinese_column=True).head().to_markdown())

    def test_fund_trend_all(self):
        data = FundTrend(fund_code="000001", period="all").load()
        data.pprint_metadata()
        print(data.to_frame(chinese_column=True))


class TestFundRant(unittest.TestCase):
    def test_fund_rant(self):
        """"""
        for period in ["1m", "3m", "6m", "1y", "3y", "5y", "this_year", "all"]:
            fund_trend = FundRant(fund_code="000746", period=period)
            data = fund_trend.load_data()
            # data.pprint_metadata()
            print(f"{period} - {data.to_frame().shape}")
            print(data.to_frame())

    def test_fund_rant_percent(self):
        """"""
        for period in ["1m", "3m", "6m", "1y", "3y", "5y", "this_year", "all"]:
            fund_trend = FundRant(fund_code="000746", percent=True, period=period)
            data = fund_trend.load_data()
            # data.pprint_metadata()
            print(f"{period} - {data.to_frame().shape}")
            print(data.to_frame())






from typing import Any, Dict, Optional, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "StockFundHolderHistoryMapping",
    "StockFundHolderHistory",
]


class StockFundHolderHistoryMapping(BaseMapping):
    """字段映射 依据股票代码查询股票基金机构对该股票的持仓历史"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "SECURITY_INNER_CODE": "证券内部代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "REPORT_DATE": "报告日期",
        "OTAL_SHARES": "总股份数",
        "HOLD_MARKET_CAP": "持有市值",
        "TOTAL_SHARES_RATIO": "总股份比例",
        "FREE_SHARES_RATIO": "自由流通股比例",
        "CHANGE_SHARES": "变动股数",
        "CHANGE_SHARES_RATIO": "变动比例"
    }


class StockFundHolderHistory(APIDataV1RequestData):
    """查询 依据股票代码查询股票基金机构对该股票的持仓历史"""

    def __init__(
            self,
            security_code: Optional[str] = None,
            start_date: Optional[str] = '2015-09-30',
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.main_holder import *

            data = StockFundHolderHistory(security_code="002475").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 股票代码
            start_date: 报告日期
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.start_date = start_date
        self.size = size
        self.mapping = StockFundHolderHistoryMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="依据股票代码查询股票基金机构对该股票的持仓历史",)
        self.conditions = []
        self.validate_security_code(code=security_code)
        self.validate_date_is_end_if_quarter(date=start_date)

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="REPORT_DATE")
        if self.security_code:
            self.conditions.append(f'(SECURITY_CODE="{self.security_code}")')
        else:
            raise
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_MAINDATA_CHANGE_POSITION",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "StockValuationMapping",
    "StockValuation",
]


class StockValuationMapping(BaseMapping):
    """字段映射 个股估值"""
    columns: Dict = {
        "SECURITY_CODE": "股票编码",
        "SECUCODE": "股票编号",
        "SECURITY_NAME_ABBR": "股票简称",
        "ORG_CODE": "机构编码",
        "TRADE_MARKET": "市场编码",
        "BOARD_CODE": "行业编码",
        "BOARD_NAME": "行业",
        "ORIG_BOARD_CODE": "行业编号",
        "TOTAL_MARKET_CAP": "总市值",
        "NOTsizeED_MARKETCAP_A": "无限制市值",
        "CLOSE_PRICE": "最新价",
        "CHANGE_RATE": "涨跌幅(%)",
        "TOTAL_SHARES": "总股份数",
        "FREE_SHARES_A": "自由流通股份",
        "PE_TTM": "PE(TTM)",
        "PE_LAR": "PE(静)",
        "PB_MRQ": "市净率",
        "PCF_OCF_LAR": "现金流比率",
        "PCF_OCF_TTM": "市现率",
        "PS_TTM": "市销率",
        "PEG_CAR": "PEG值",
        "TRADE_DATE": "交易日期"
    }


class StockValuation(APIDataV1RequestData):
    """查询 个股估值 """
    def __init__(
            self,
            security_code: Optional[str] = None,
            industry_code: Optional[str] = None,
            date: Optional[str] = None,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            # 1. 查询全部股票的估值信息
            data = StockValuation(date="2024-09-30").load()
            data.to_frame(chinese_column=True)

            # 2. 查询某行业全部股票的估值信息
            data = StockValuation(date="2024-09-30", industry_code="016023").load()
            data.to_frame(chinese_column=True)

            # 3. 查询某只股票的估值信息
            data = StockValuation(start_data="2024-08-30", code="000059").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            industry_code: 行业代码
            date: 交易日期
            start_date: 开始日期
            end_date: 结束日期
            size: 返回条数
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.industry_code = industry_code
        self.date = date
        self.start_date = start_date
        self.end_date = end_date
        self.size = size
        self.mapping = StockValuationMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="个股估值")
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date, date])

    def params_filter(self) -> str:
        """
        :return:
        """
        self.filter_report_date(date_name="TRADE_DATE")
        if self.industry_code:
            self.validate_code(code=self.industry_code, length=6)
            self.conditions.append(f"(BOARD_CODE=\"{self.industry_code}\")")
        if self.security_code:
            self.validate_code(code=self.security_code, length=6)
            self.conditions.append(f"(SECURITY_CODE=\"{self.security_code}\")")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "pageSize": self.size,
            "reportName": "RPT_VALUEANALYSIS_DET",
            "sortColumns": "TRADE_DATE,SECURITY_CODE",
            "sortTypes": "-1,1",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



import unittest
from wisecon.fund.fund_hist import FundHist


class TestFundList(unittest.TestCase):
    def test_fund_hist(self):
        """"""
        data = FundHist(fund_code="000746").load()
        print(data.to_frame().head().to_markdown())
        print(data.to_frame(data.metadata.get("columns")).head().to_markdown())

    def test_fund_hist_date(self):
        """"""
        data = FundHist(fund_code="000746", start_date="2020-01-01", end_date="2024-01-01", limit=10).load()
        print(data.to_frame().shape)
        print(data.to_frame(chinese_column=True).head().to_markdown())



import unittest
from wisecon.stock.new_ipo import *


class TestNewIPO(unittest.TestCase):

    def test_new_ipo_stock_all(self):
        data = NewIPOList(market="全部股票", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_sh(self):
        data = NewIPOList(market="沪市主板", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_kc(self):
        data = NewIPOList(market="科创板", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_sz(self):
        data = NewIPOList(market="深市主板", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_cy(self):
        data = NewIPOList(market="创业板", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_bj(self):
        data = NewIPOList(market="北交所", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_zai(self):
        data = NewIPOList(market="可转债", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_new_ipo_stock_reit(self):
        data = NewIPOList(market="REITs", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())



import traceback
from pydantic import BaseModel, Field
from typing import Any, List, Dict, Optional, Callable
from wisecon.types.agent import Function
from wisecon.agent.tools.register import register_tool


__all__ = [
    "Tools",
]


class Tools(BaseModel):
    """"""
    descriptions: List[Dict] = Field(default=[], description="Description of the tool")
    hooks: Dict[str, Callable] = Field(default={}, description="Hooks of the tool")
    tools: List[Callable] = Field(default=None, description="List of function to be used as tools")
    params_fun: Optional[Callable] = Field(default=None, description="Function to clean the parameters of the tool")

    def __init__(self, tools: List[Callable], **kwargs):
        super().__init__(tools=tools, **kwargs)
        self.register_tools()

    def register_tools(self):
        """"""
        _ = [register_tool(hooks=self.hooks, descriptions=self.descriptions)(fun)for fun in self.tools]

    def dispatch(self, function: Function) -> Any:
        """"""
        function = Function.model_validate(function.model_dump())
        if function.name not in self.hooks:
            return f"Tool `{function.name}` not found. Please use a provided tool."
        tool_call = self.hooks[function.name]
        try:
            data = tool_call(**function.arguments).load()
            ret = data.to_frame(chinese_column=True).to_markdown()
        except:
            ret = traceback.format_exc()
        return ret



import requests
import pandas as pd
from requests import Response
from tqdm import tqdm
from typing import List, Dict, Optional

from wisecon.types import ResponseData
from wisecon.types.request_data import BaseRequestData


__all__ = [
    "APICurrentCarRequestData",
    "APICarHistoryRequestData",
]


class APICurrentCarRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://xl.16888.com/xl.php"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "mod": "api",
            "extra": "factoryRanking",
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.get("data", {})
        data = response.pop("list")
        self.metadata.response = response
        return data


class APICarHistoryRequestData(BaseRequestData):
    """"""
    data_mark: Optional[str]
    page: Optional[int]
    data_page_name: Optional[str]
    multi_pages: Optional[bool]

    def base_url(self, **kwargs) -> str:
        return f"https://xl.16888.com/{self.data_page_name}.html"

    def request(self, **kwargs) -> Response:
        """"""
        base_url = self.base_url(**kwargs)
        self._logger(msg=f"URL: {base_url}\n", color="green")
        response = requests.get(base_url, headers=self.headers)
        return response

    def request_data(self, df_list: List) -> bool:
        """"""
        self.data_page_name = f"{self.data_mark}-{self.page}"
        content = self.request().content
        df = pd.read_html(content)[0]
        if len(df) > 1:
            df_list.append(df)
            self.page += 1
            mark = True
        else:
            mark = False
        return mark

    def request_html(self) -> List[Dict]:
        """"""
        self.data_page_name = self.data_mark
        content = self.request().content
        df = pd.read_html(content)[0]
        return df.to_dict(orient="records")

    def request_mutil_html(self, ) -> List[Dict]:
        self.page = 1
        mark = True
        df_list = []

        if self.verbose:
            with tqdm() as pbar:
                while mark:
                    mark = self.request_data(df_list)
                    pbar.update(1)
        else:
            while mark:
                mark = self.request_data(df_list)
        df = pd.concat(df_list, ignore_index=True)
        return df.to_dict(orient="records")

    def load(self) -> ResponseData:
        """"""
        if self.response_type == "html":
            if self.multi_pages:
                data = self.request_mutil_html()
            else:
                data = self.request_html()
        else:
            raise NotImplementedError("Not implemented yet")
        return self.data(data=data, metadata=self.metadata)



import os
import time
import requests
from typing import Any, List, Dict, Union, Callable, Optional
from wisecon.types import BaseMapping
from wisecon.types.request_api.report import *
from wisecon.utils import tqdm_progress_bar
from lumix.documents import StructuredPDF

__all__ = [
    "Report",
]


class ReportMapping(BaseMapping):
    """"""
    columns: Dict = {
        "title": "标题",
        "stockName": "股票名称",
        "stockCode": "股票代码",
        "orgCode": "机构代码",
        "orgName": "机构名称",
        "orgSName": "机构简称",
        "publishDate": "发布日期",
        "infoCode": "信息编码",
        "column": "栏目",
        "predictNextTwoYearEps": "预测未来两年每股收益",
        "predictNextTwoYearPe": "预测未来两年市盈率",
        "predictNextYearEps": "预测下一年每股收益",
        "predictNextYearPe": "预测下一年市盈率",
        "predictThisYearEps": "预测今年每股收益",
        "predictThisYearPe": "预测今年市盈率",
        "predictLastYearEps": "预测去年每股收益",
        "predictLastYearPe": "预测去年市盈率",
        "actualLastTwoYearEps": "实际过去两年每股收益",
        "actualLastYearEps": "实际去年每股收益",
        "industryCode": "行业编码",
        "industryName": "行业名称",
        "emIndustryCode": "细分行业编码",
        "indvInduCode": "个体行业编码",
        "indvInduName": "个体行业名称",
        "emRatingCode": "评级编码",
        "emRatingValue": "评级值",
        "emRatingName": "评级名称",
        "lastEmRatingCode": "最近评级编码",
        "lastEmRatingValue": "最近评级值",
        "lastEmRatingName": "最近评级名称",
        "ratingChange": "评级变动",
        "reportType": "报告类型",
        "author": "作者",
        "indvIsNew": "个体是否为新",
        "researcher": "研究员",
        "newListingDate": "新上市日期",
        "newPurchaseDate": "新购买日期",
        "newIssuePrice": "新发行价格",
        "newPeIssueA": "新市盈率发行A",
        "indvAimPriceT": "个体目标价格T",
        "indvAimPriceL": "个体目标价格L",
        "attachType": "附件类型",
        "attachSize": "附件大小",
        "attachPages": "附件页数",
        "encodeUrl": "编码网址",
        "sRatingName": "综合评级名称",
        "sRatingCode": "综合评级编码",
        "market": "市场",
        "authorID": "作者ID",
        "count": "计数",
        "orgType": "机构类型"
    }


class Report(APIReportRequest):
    """"""
    def __init__(
            self,
            code: Optional[str] = None,
            industry: Optional[str] = "*",
            industry_code: Optional[Union[str, int]] = "*",
            size: Optional[int] = 5,
            rating: Optional[str] = "*",
            rating_change: Optional[str] = "*",
            begin_time: Optional[str] = "*",
            end_time: Optional[str] = "*",
            page_no: Optional[int] = 1,
            report_type: Optional[TypeReport] = "*",
            q_type: Optional[Union[int, str]] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """

        Args:
            code: 股票代码
            industry:
            industry_code:
            size:
            rating:
            rating_change:
            begin_time:
            end_time:
            page_no:
            report_type:
            q_type:
            verbose:
            logger:
            **kwargs:
        """
        self.code = code
        self.industry = industry
        self.industry_code = industry_code
        self.size = size
        self.rating = rating
        self.rating_change = rating_change
        self.begin_time = begin_time
        self.end_time = end_time
        self.page_no = page_no
        self.report_type = report_type
        self.q_type = q_type
        self.mapping = ReportMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="投研报告",
        )

    def params(self) -> Dict:
        """"""
        if self.q_type is None:
            self.reports_type()

        params = {
            "industry": self.industry,
            "industryCode": self.industry_code,
            "pageSize": self.size,
            "rating": self.rating,
            "ratingChange": self.rating_change,
            "beginTime": self.begin_time,
            "endTime": self.end_time,
            "pageNo": self.page_no,
            "qType": self.q_type,
            "_": self.current_time()
        }
        if self.code:
            params.update({"code": self.code})
        return self.base_param(update=params)

    def bytes_file(
            self,
            info_codes: List[str]
    ):
        """"""
        self.reports_data = []
        base_url = """https://pdf.dfcfw.com/pdf/H3_{}_1.pdf""".format
        for info_code in tqdm_progress_bar(info_codes):
            _report = ReportData(code=info_code)
            try:
                response = requests.get(base_url(info_code), headers=self.headers)
                _report.content = response.content
            except Exception as e:
                self._logger(msg=f"[{__class__.__name__}] Load `{info_code}` error, error message: {e}", color="red")
                _report.error = str(e)
            self.reports_data.append(_report)

    def to_bytes_content(self, info_code: str) -> bytes:
        """"""
        base_url = f"""https://pdf.dfcfw.com/pdf/H3_{info_code}_1.pdf?1746631765000.pdf"""
        try:
            response = requests.get(base_url, headers=self.headers)
            return response.content
        except Exception as e:
            msg = f"[{__class__.__name__}] Load `{info_code}` error, error message: {e}"
            self._logger(msg=msg, color="red")
            raise Exception(msg)

    def to_text(self, info_code: str) -> str:
        """"""
        bytes_data = self.to_bytes_content(info_code=info_code)
        pdf = StructuredPDF(path_or_data=bytes_data)
        return pdf.to_text()

    def save_pdf(self, info_code: str, path: Optional[str] = None):
        """"""
        if path is None:
            path = f"{info_code}.pdf"
        bytes_data = self.to_bytes_content(info_code=info_code)
        with open(path, "wb") as f:
            f.write(bytes_data)

    def save(
            self,
            path: str = "./reports",
            info_codes: Optional[List[str]] = None,
    ):
        """"""
        if not os.path.exists(path):
            os.makedirs(path)
        cache_path = os.path.join(path, "cache")
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)
        pdf_path = cache_path = os.path.join(path, "pdf")
        if not os.path.exists(pdf_path):
            os.makedirs(pdf_path)

        if info_codes is None:
            response_data = self.load()
            if len(response_data.data) > 0:
                report_data = response_data.to_frame()
                info_codes = report_data["infoCode"].tolist()
                report_data.to_csv(os.path.join(cache_path, f"{str(int(time.time() * 1E3))}.csv"), index=False)
            else:
                info_codes = []
                self._logger(msg=f"[{__class__.__name__}] Not find report.")
        if len(info_codes) > 0:
            self.bytes_file(info_codes=info_codes)
            for _report in self.reports_data:
                if isinstance(_report.content, bytes):
                    file_path = os.path.join(pdf_path, f"{_report.code}.pdf")
                    with open(file_path, "wb") as f:
                        f.write(_report.content)


def report_info(
            code: Optional[str] = None,
            industry: Optional[str] = "*",
            industry_code: Optional[Union[str, int]] = "*",
            size: Optional[int] = 5,
            rating: Optional[str] = "*",
            rating_change: Optional[str] = "*",
            begin_time: Optional[str] = "*",
            end_time: Optional[str] = "*",
            page_no: Optional[int] = 1,
            report_type: Optional[TypeReport] = "*",
            q_type: Optional[Union[int, str]] = None,
):
    """"""
    report = Report(
        code=code, industry=industry, industry_code=industry_code, size=size,
        rating=rating, rating_change=rating_change, begin_time=begin_time,
        end_time=end_time, page_no=page_no, report_type=report_type, q_type=q_type
    )
    columns = [

    ]
    data = report.load().to_frame(chinese_column=True)
    return data.to_markdown(index=False)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "DepositRateMapping",
    "DepositRate",
]


class DepositRateMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "公布时间",
        "PUBLISH_DATE": "生效时间",
        "DEPOSIT_RATE_BB": "存款基准利率(调整前)",
        "DEPOSIT_RATE_BA": "存款基准利率(调整后)",
        "DEPOSIT_RATE_B": "存款基准利率(调整幅度)",
        "LOAN_RATE_SB": "贷款基准利率(调整前)",
        "LOAN_RATE_SA": "贷款基准利率(调整后)",
        "LOAN_RATE_S": "贷款基准利率(调整幅度)",
        "NEXT_SH_RATE": "消息公布次日指数涨跌(SH)",
        "NEXT_SZ_RATE": "消息公布次日指数涨跌(SZ)",
    }


class DepositRate(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = DepositRateMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 利率调整",
        )

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "PUBLISH_DATE", "DEPOSIT_RATE_BB", "DEPOSIT_RATE_BA", "DEPOSIT_RATE_B", "LOAN_RATE_SB",
            "LOAN_RATE_SA", "LOAN_RATE_S", "NEXT_SH_RATE", "NEXT_SZ_RATE",
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_DEPOSIT_RATE",
        }
        return self.base_param(update=params)



import unittest
from wisecon.stock.kline import *


class TestKLine(unittest.TestCase):

    def test_300(self):
        """"""
        data = KLine(market_code="000300", period="1D", size=5, verbose=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_stock(self):
        """"""
        # data = KLine(security_code="300069", period="1D", size=5, verbose=True).load()
        # data = KLine(security_code="601939", period="1D", size=5, verbose=True).load()
        data = KLine(security_code="002006", period="1D", size=5, verbose=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_plate(self):
        """"""
        data = KLine(plate_code="BK0887", period="1D", size=5, verbose=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_loop_period(self):
        """"""
        for period in ["1m", "5m", "15m", "30m", "60m", "1D", "1W", "1M"]:
            data = KLine(security_code="300069", period=period, size=5).load()
            print(period, data.to_frame(chinese_column=True).shape)


class TestMinKLine(unittest.TestCase):
    def test_1min(self):
        """"""
        data = KlineMin(security_code="300069", n_days=1, verbose=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_plate(self):
        """"""
        data = KlineMin(plate_code="BK0887", n_days=1, verbose=True).load()
        print(data.to_markdown(chinese_column=True))



from typing import Any, Dict, Optional, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "FundHolderListMapping",
    "FundHolderList",
]


class FundHolderListMapping(BaseMapping):
    """字段映射 基金持仓数据"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "SECURITY_INNER_CODE": "证券内部代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "REPORT_DATE": "报告日期",
        "HOLDER_CODE": "持有人代码",
        "HOLDER_NAME": "持有人名称",
        "PARENT_ORG_CODE": "母组织代码",
        "PARENT_ORGCODE_OLD": "旧母组织代码",
        "PARENT_ORG_NAME": "母组织名称",
        "ORG_TYPE_CODE": "组织类型代码",
        "ORG_TYPE": "组织类型",
        "TOTAL_SHARES": "总持股数",
        "HOLD_MARKET_CAP": "持有市值",
        "TOTAL_SHARES_RATIO": "总持股比例",
        "FREE_SHARES_RATIO": "自由流通股比例",
        "NETASSET_RATIO": "净资产比例",
        "ORG_NAME_ABBR": "组织名称简称"
    }


class FundHolderList(APIDataV1RequestData):
    """查询 依据基金代码查询基金持仓数据"""

    def __init__(
            self,
            holder_code: Optional[str] = None,
            date: Optional[str] = None,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.main_holder import *

            data = FundHolderList(holder_code="516060", date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder_code: `持有组织代码/基金代码`
            date: 报告日期
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.holder_code = holder_code
        self.date = date
        self.size = size
        self.mapping = FundHolderListMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="基金持仓数据",)
        self.conditions = []
        self.validate_holder_code(code=holder_code)
        self.validate_date_is_end_if_quarter(date=date)

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="REPORT_DATE")
        if self.holder_code:
            self.conditions.append(f'(HOLDER_CODE="{self.holder_code}")')
        else:
            raise
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "SECURITY_CODE",
            "sortTypes": "-1",
            "reportName": "RPT_MAINDATA_MAIN_POSITIONDETAILS",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "FreeHolderStockAnalysis",
    "FreeHolderStockAnalysisMapping",
]


class FreeHolderStockAnalysisMapping(BaseMapping):
    """字段映射 上市公司十大流通股东股东持股统计"""
    columns: Dict = {
        "COOPERATION_HOLDER_MARK": "合作股东标识",
        "END_DATE": "截止日期",
        "HOLDER_NAME": "股东名称",
        "HOLDER_TYPE": "股东类型",
        "HOLDNUM_CHANGE_TYPE": "持股变动类型",
        "STATISTICS_TIMES": "统计次数",
        "AVG_CHANGE_10TD": "10日平均变动",
        "MAX_CHANGE_10TD": "10日最大变动",
        "MIN_CHANGE_10TD": "10日最小变动",
        "AVG_CHANGE_30TD": "30日平均变动",
        "MAX_CHANGE_30TD": "30日最大变动",
        "MIN_CHANGE_30TD": "30日最小变动",
        "AVG_CHANGE_60TD": "60日平均变动",
        "MAX_CHANGE_60TD": "60日最大变动",
        "MIN_CHANGE_60TD": "60日最小变动",
        "SEAB_JOIN": "关联股票"
    }


class FreeHolderStockAnalysis(APIDataV1RequestData):
    """查询 上市公司十大流通股东股东持股统计"""
    def __init__(
            self,
            holder_name: Optional[str] = None,
            holder_type: Optional[Literal["个人", "基金", "QFII", "社保", "券商", "信托"]] = None,
            holder_change: Optional[Literal["新进", "增加", "不变", "减少"]] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.holder import *

            data = FreeHolderStockAnalysis(size=20, start_date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder_name: 股东名称
            holder_type: 股东类型 `["个人", "基金", "QFII", "社保", "券商", "信托"]`
            holder_change: 变动方向 `["新进", "增加", "不变", "减少"]`
            size: 返回条数
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否显示日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.holder_name = holder_name
        self.holder_type = holder_type
        self.holder_change = holder_change
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = FreeHolderStockAnalysisMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司十大流通股东股东持股统计")
        self.conditions = []

    def params_hold_change(self):
        """"""
        change_mapping = {"新进": "002", "增加": "003", "不变": "004", "减少": "005"}
        if self.holder_change:
            return change_mapping.get(self.holder_change, "001")
        else:
            return "001"

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="END_DATE")
        if self.holder_type:
            self.conditions.append(f'(HOLDER_TYPE="{self.holder_type}")')
        if self.holder_change:
            self.conditions.append(f'(HOLDNUM_CHANGE_TYPE="{self.params_hold_change()}")')
        if self.holder_name:
            self.conditions.append(f'(HOLDER_NAME+like+"%{self.holder_name}%")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "STATISTICS_TIMES,COOPERATION_HOLDER_MARK",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_COOPFREEHOLDERS_ANALYSISNEW",
        }
        return self.base_param(params)



import unittest
from wisecon.report import *
from wisecon.mcp.report import *
import pandas as pd


class TestReport(unittest.TestCase):
    def test_stock_morning_news_report(self):
        """"""
        columns = ["reportType", "columnType", "title"]
        report = Report(report_type="券商晨报", size=100, begin_time="2024-09-23", end_time="2024-09-23")
        data = report.load()
        print(data.to_frame()[columns].to_markdown())

    def test_strategy_report(self):
        """"""
        columns = ["reportType", "columnType", "title"]
        report = Report(report_type="策略报告", size=50, begin_time="2024-09-23", end_time="2024-09-23")
        data = report.load()
        print(data.to_frame()[columns].to_markdown())

    def test_macro_report(self):
        """"""
        columns = ["reportType", "columnType", "title"]
        report = Report(report_type="宏观研究", size=5, begin_time="2024-09-23", end_time="2024-09-23")
        data = report.load()
        # print(data.to_frame()[columns].to_markdown())
        columns = ["title", "orgSName", "infoCode"]
        print(data.to_frame().to_markdown())

    def test_industry_report(self):
        """"""
        columns = ["industryName", "industryCode", "reportType", "title"]
        report = Report(report_type="行业研报", size=5, begin_time="2024-09-23", end_time="2024-09-23", verbose=True)
        data = report.load()
        print(data.to_frame()[columns].to_markdown())

    def test_industry_code_report(self):
        """"""
        columns = ["industryName", "industryCode", "reportType", "title"]
        report = Report(report_type="行业研报", industry_code="451", size=5, begin_time="2024-09-23", end_time="2024-09-23", verbose=True)
        data = report.load()
        # print(data.to_frame()[columns].to_markdown())
        columns = ["title", "orgSName", "infoCode"]
        print(data.to_frame()[columns].to_markdown())

    def test_stock_report(self):
        """"""
        columns = ["stockCode", "stockName", "reportType", "title"]
        report = Report(report_type="个股研报", size=500, begin_time="2024-09-23", end_time="2024-09-23", verbose=True)
        data = report.load()
        print(data.to_frame()[columns].to_markdown())

    def test_stock_code_report(self):
        """"""
        columns = ["stockCode", "stockName", "reportType", "title"]
        report = Report(code="002222", report_type="个股研报", size=500, begin_time="2024-09-23", end_time="2024-09-23", verbose=True)
        data = report.load()
        print(data.to_frame()[columns].to_markdown())

    def test_save_report(self):
        """"""
        report = Report(
            report_type="行业研报", industry_code="451", size=5, begin_time="2024-09-23",
            end_time="2024-09-23")
        content = report.to_text(info_code="AP202409231639991919")
        print(content)
        report.save_pdf(info_code="AP202409231639991919")


class TestReportMapping(unittest.TestCase):
    """"""
    def test_report_mapping(self):
        """"""
        con_map = ConceptionMap()
        print(con_map.map_district)

    def test_get_name(self):
        """"""
        con_map = ConceptionMap()
        print(con_map.get_code_by_name(name="玻璃"))

    def test_get_code(self):
        """"""
        con_map = ConceptionMap()
        print(con_map.get_name_by_code(code="451"))

    def test_range_report(self):
        """"""
        date = pd.date_range(start="2025-01-11", end="2025-02-14").astype(str).to_list()
        for d in date:
            print(d)
            report = Report(verbose=True, begin_time=d, end_time=d, size=100)
            report.save(path="/Users/chensy/Desktop/reports")


class TestMCPReport(unittest.TestCase):
    """"""
    def test_get_now_date(self):
        """"""
        print(get_now_date())

    def test_get_industry_name_by_code(self):
        """"""
        data = get_industry_name_by_code(code="451")
        print(data)

    def test_get_industry_code_by_name(self):
        """"""
        data = get_industry_code_by_name("玻璃")
        print(data)

    def test_list_report(self):
        """"""
        data = list_industry()
        print(data)

    def test_fetch_report_text(self):
        """"""
        data = fetch_report_text_by_code(**{"info_code": "AP202505061668519723"})
        print(data)

    def test_fetch_report_text_by_code_error(self):
        data = fetch_report_text_by_code(info_code="-asd919")
        print(data)

# class TestProfitForecast(unittest.TestCase):
#
#     def test_forecast(self):
#         """"""
#         report = ProfitForecast(size=2)
#         data = report.load_data()
#         columns = data.metadata.get("columns")
#         print(data.to_frame(columns=columns).to_markdown())
#
#     def test_forecast_by_industry(self):
#         """"""
#         report = ProfitForecast(industry="电源设备", size=2)
#         data = report.load_data()
#         columns = data.metadata.get("columns")
#         print(data.to_frame(columns=columns).to_markdown())
#
#     def test_forecast_by_conception(self):
#         """"""
#         report = ProfitForecast(conception="高压快充", size=2)
#         data = report.load_data()
#         columns = data.metadata.get("columns")
#         print(data.to_frame(columns=columns).to_markdown())
#
#     def test_forecast_by_district(self):
#         """"""
#         report = ProfitForecast(district="北京板块", size=2)
#         data = report.load_data()
#         columns = data.metadata.get("columns")
#         print(data.to_frame(columns=columns).to_markdown())



from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "DepartmentActiveMapping",
    "DepartmentActive",
]


class DepartmentActiveMapping(BaseMapping):
    """字段映射 每日活跃营业部"""
    columns: Dict = {
        "OPERATEDEPT_NAME": "营业部名称",
        "ONLIST_DATE": "上榜日期",
        "BUYER_APPEAR_NUM": "买方出现次数",
        "SELLER_APPEAR_NUM": "卖方出现次数",
        "TOTAL_BUYAMT": "总买入金额",
        "TOTAL_SELLAMT": "总卖出金额",
        "TOTAL_NETAMT": "净金额",
        "BUY_STOCK": "买入股票",
        "OPERATEDEPT_CODE": "营业部代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "OPERATEDEPT_CODE_OLD": "旧营业部代码",
        "ORG_NAME_ABBR": "组织名称简称"
    }


class DepartmentActive(APIDataV1RequestData):
    """查询 每日活跃营业部"""

    def __init__(
            self,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = DepartmentActive(date="2024-10-28").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            start_date: 开始日期, 格式: yyyy-MM-dd, 默认值: None
            end_date: 结束日期, 格式: yyyy-MM-dd, 默认值: None
            date: 日期, 格式: yyyy-MM-dd, 默认值: None
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.size = size
        self.mapping = DepartmentActiveMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="每日活跃营业部",)
        self.conditions = []
        self.validate_date_format(date=[date, start_date, end_date])

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="ONLIST_DATE")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "TOTAL_NETAMT,ONLIST_DATE,OPERATEDEPT_CODE",
            "sortTypes": "-1,-1,1",
            "reportName": "RPT_OPERATEDEPT_ACTIVE",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIStockFFlowKLineRequestData


__all__ = [
    "CapitalFlowMinutesMapping",
    "CapitalFlowMinutes",
]


TypeMarket = Literal["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]


class CapitalFlowMinutesMapping(BaseMapping):
    """字段映射 市场、板块（行业、概念、地区）分钟级资金流向数据"""
    columns: Dict = {
        "TRADE_DATE": "时间",
        "main_inflow": "主力净流入",
        "small_inflow": "小单净流入",
        "medium_inflow": "中单净流入",
        "large_inflow": "大单净流入",
        "block_inflow": "超大单净流入",
    }


class CapitalFlowMinutes(APIStockFFlowKLineRequestData):
    """查询 市场、板块（行业、概念、地区）分钟级资金流向数据

    1. 查询市场总体资金流量
    2. 查询全部 `行业、地区、概念` 板块资金流量
    3. 查询指定板块资金流量
    4. 查询不同行业、地区、概念板块下的主力流入流出排名

    Notes:
        ```markdown
        指标定义
        　　- 超大单：大于等于50万股或者100万元的成交单;
        　　- 大单：大于等于10万股或者20万元且小于50万股和100万元的成交单;
        　　- 中单：大于等于2万股或者4万元且小于10万股和20万元的成交单;
        　　- 小单：小于2万股和4万元的成交单;
        ```
    """
    def __init__(
            self,
            market: Optional[TypeMarket] = None,
            plate_code: Optional[str] = None,
            security_code: Optional[str] = None,
            size: Optional[int] = 0,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.capital_flow import *

            # 1. 查询市场的资金分钟流向数据
            data = CapitalFlowMinutes(market="沪深两市", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.1 查询板块的资金分钟流向数据(概念)
            data = CapitalFlowMinutes(plate_code="BK1044", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.2 查询板块的资金分钟流向数据(地区)
            data = CapitalFlowMinutes(plate_code="BK0158", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.3 查询板块的资金分钟流向数据(行业)
            data = CapitalFlowMinutes(plate_code="BK1044", size=10).load()
            data.to_frame(chinese_column=True)

            # 3. 查询股票的资金分钟流向数据
            data = CapitalFlowMinutes(security_code="300750", size=10).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场名称: `["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]`
            plate_code: 板块代码, 支持地区板块、行业板块、概念板块
            security_code: 股票代码
            size: 数据条数
            verbose: 是否打印日志
            logger: 自定义日志函数
            **kwargs: 其他参数
        """
        self.market = market
        self.plate_code = plate_code
        self.security_code = security_code
        self.size = size
        self.mapping = CapitalFlowMinutesMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="市场、板块（行业、概念、地区）分钟级资金流向数据", )
    
    def params_market_code(self) -> Dict:
        """"""
        market_mapping = {
            "沪深两市": "1.000001,0.399001", "沪市": "1.000001", "深市": "0.399001", "创业板": "0.399006", "沪B": "1.000003", "深B": "0.399003"
        }
        if self.market in market_mapping:
            if self.market == "沪深两市":
                secid, secid2 = market_mapping["沪深两市"].split(",")
                return {"secid": secid, "secid2": secid2}
            else:
                return {"secid": market_mapping[self.market]}
        else:
            raise ValueError(
                f'market error {self.market} not in ["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]')

    def params_plate_code(self) -> Dict:
        """"""
        return {"secid": f"90.{self.plate_code}"}

    def params_security_code(self) -> Dict:
        """"""
        return {"secid": f"0.{self.security_code}"}
    
    def params_secid(self) -> Dict:
        """"""
        if self.market:
            return self.params_market_code()
        elif self.plate_code:
            return self.params_plate_code()
        elif self.security_code:
            return self.params_security_code()
        else:
            raise ValueError("market_code or plate_code must be set")

    def params(self) -> Dict:
        """"""
        param_sec_id = self.params_secid()
        params = {
            "klt": "1",
            "lmt": self.size,
            "fields1": "f1,f2,f3,f7",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
        }
        params.update(param_sec_id)
        return self.base_param(update=params)



import time
import random


__all__ = [
    "headers",
    "Constant",
    "get_current_timestamp",
    "jquery_mock_callback",
]


headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36',
}


class Constant:
    jQuery_Version = "1.8.3"


def get_current_timestamp() -> int:
    return int(round(time.time() * 1000))


def jquery_mock_callback() -> str:
    return f'jQuery{(Constant.jQuery_Version + str(random.random())).replace(".", "")}_{str(get_current_timestamp() - 1000)}'



from typing import Any, List, Dict, Union, Literal, Callable, Optional, Annotated
from .base import CapitalFlowCurrentBaseMapping, CapitalFlowCurrentRequestData


__all__ = [
    "CapitalFlowCurrentMapping",
    "CapitalFlowCurrent",
]


TypeMarket = Literal["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]


class CapitalFlowCurrentMapping(CapitalFlowCurrentBaseMapping):
    """字段映射 当前股票资金流量统计"""


class CapitalFlowCurrent(CapitalFlowCurrentRequestData):
    """查询 当前股票资金流量统计，最多可以同步查询10条，超过10条请使用 `StockFlow` 方法。

    1. 查询沪深两市市场当前资金动向数据
    2. 查询某行业、地区、概念板块当前资金动向数据
    3. 查询具体股票的当前资金动向数据
    """
    def __init__(
            self,
            market: Annotated[Optional[TypeMarket], "市场类型", False] = None,
            plate_code: Annotated[Optional[Union[str, List[str]]], "板块代码", False] = None,
            security_code: Annotated[Optional[Union[str, List[str]]], "证券代码", False] = None,
            size: Annotated[Optional[int], "数据量大小", False] = 10,
            verbose: Annotated[Optional[bool], "是否打印日志", False] = False,
            logger: Annotated[Optional[Callable], "日志打印函数", False] = None,
            **kwargs: Annotated[Any, "其他参数", False],
    ):
        """
        查询 当前股票资金流量统计，最多可以同步查询10条，超过10条请使用 `StockFlow` 方法。

        1. 查询沪深两市市场当前资金动向数据
        2. 查询某行业、地区、概念板块当前资金动向数据
        3. 查询具体股票的当前资金动向数据

        Notes:
            ```python
            from wisecon.stock.capital_flow import *

            # 1. 查询股票的资金当前流向数据
            data = CapitalFlowCurrent(security_code="300750", size=10).load()
            data.to_frame(chinese_column=True)

            # 2. 查询板块的资金当前流向数据
            data = CapitalFlowCurrent(plate_code="BK0477", size=10).load()
            data.to_frame(chinese_column=True)

            # 3. 查询市场的资金当前流向数据
            data = CapitalFlowCurrent(market="深市A股", size=10).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场名称 `["全部股票", "沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "沪市B股", "深市B股"]`
            plate_code: 板块代码, 行业、概念、地区
            security_code: 股票代码
            size: 数据条数
            verbose: 是否显示日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.market = market
        self.plate_code = plate_code
        self.security_code = security_code
        self.size = size
        self.mapping = CapitalFlowCurrentMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="当前股票资金流量统计", )
        self.validate_max_security_codes(security_code=security_code)

    def params_market_code(self) -> str:
        """"""
        market_mapping = {
            "沪深两市": "1.000001,0.399001", "沪市": "1.000001", "深市": "0.399001",
            "创业板": "0.399006", "沪B": "1.000003", "深B": "0.399003"
        }
        if self.market in market_mapping:
            return market_mapping[self.market]
        else:
            raise ValueError(
                f'market error {self.market} not in ["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]')

    def params_security_code(self) -> str:
        """"""
        if isinstance(self.security_code, str):
            self.security_code = [self.security_code]
        for i, code in enumerate(self.security_code):
            if code.startswith("3"):
                self.security_code[i] = f"0.{code}"
            else:
                self.security_code[i] = f"1.{code}"
        return ",".join(self.security_code)

    def params_plate_code(self) -> str:
        """"""
        if isinstance(self.plate_code, str):
            self.plate_code = [self.plate_code]
        return ",".join([f"90.{code}" for code in self.plate_code])

    def params_secids(self) -> str:
        """"""
        if self.market is not None:
            return self.params_market_code()
        elif self.security_code is not None:
            return self.params_security_code()
        elif self.plate_code is not None:
            return self.params_plate_code()
        else:
            raise ValueError("security_code or plate_code must be set.")

    def params(self) -> Dict:
        """
        :return:
        """
        params = {"secids": self.params_secids()}
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "IndustryValuationMapping",
    "IndustryValuation",
]


class IndustryValuationMapping(BaseMapping):
    """字段映射 上市公司行业估值"""
    columns: Dict = {
        "BOARD_NAME": "行业",
        "BOARD_CODE": "行业代码",
        "ORIGINALCODE": "行业编码",
        "TRADE_DATE": "交易日期",
        "PE_TTM": "PE(TTM)",
        "PE_LAR": "PE(静)",
        "PB_MRQ": "市净率",
        "PCF_OCF_TTM": "市现率",
        "PS_TTM": "市销率",
        "PEG_CAR": "PEG值",
        "TOTAL_MARKET_CAP": "总市值",
        "MARKET_CAP_VAG": "平均市值",
        "NOTLIMITED_MARKETCAP_A": "非限制市值",
        "NOMARKETCAP_A_VAG": "无市场价值的可变资产",
        "TOTAL_SHARES": "总股份",
        "TOTAL_SHARES_VAG": "可变股份总数",
        "FREE_SHARES_VAG": "平均市值",
        "NUM": "个股数量",
        "LOSS_COUNT": "亏损家数"
    }


class IndustryValuation(APIDataV1RequestData):
    """查询 上市公司行业估值"""
    def __init__(
            self,
            industry_code: Optional[str] = None,
            date: Optional[str] = None,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            # 1. 查询某日期全部行业估值
            data = IndustryValuation(date="2024-10-10").load()
            data.to_frame(chinese_column=True)

            # 2. 查询某周期某行业估值
            data = IndustryValuation(
                start_date="2024-10-01", end_date="2024-10-20",
                limit=50, industry_code="016017"
            ).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            industry_code: 行业代码
            date: 查询日期
            start_date: 开始日期
            end_date: 结束日期
            size: 返回条数
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.industry_code = industry_code
        self.date = date
        self.start_date = start_date
        self.end_date = end_date
        self.size = size
        self.mapping = IndustryValuationMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date, date], )
        self.request_set(description="上市公司行业估值")

    def params_industry_code(self):
        """"""
        if self.industry_code:
            self.validate_code(self.industry_code, length=6)
            self.conditions.append(f"(BOARD_CODE=\"{self.industry_code}\")")

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="TRADE_DATE")
        self.params_industry_code()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "pageSize": self.size,
            "reportName": "RPT_VALUEINDUSTRY_DET",
            "pageNumber": 1,
            "sortColumns": "TRADE_DATE,PE_TTM",
            "sortTypes": "-1,1",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



import unittest
from wisecon.fund.fund_company import FundCompany


class TestFundList(unittest.TestCase):
    def test_fund_hist(self):
        """"""
        data = FundCompany(verbose=True).load()
        data.pprint_metadata()
        print(data.to_frame().head().to_markdown())
        print(data.to_frame(chinese_column=True).head().to_markdown())



import unittest
from wisecon.stock.ranking import *


class TestStockRanking(unittest.TestCase):
    """
    机构席位追踪每日活跃营业部营业部排行营业部统计营业部查询

    """
    def test_stock_ranking_detail(self):
        """龙虎榜详情"""
        data = StockRankDetail(date="2024-10-28").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_ranking_detail_with_market(self):
        """龙虎榜详情"""
        data = StockRankDetail(market="沪市A股", date="2024-10-28").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_ranking_statistic(self):
        """个股上榜统计"""
        data = StockRankStatistic(statistics_cycle="1m").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_institution_ranking(self):
        """机构买卖每日统计"""
        data = InstitutionTradeRank(start_date="2024-10-23").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_institution_seat(self):
        """机构席位追踪"""
        data = InstitutionSeat(statistics_cycle="1m").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_department_active(self):
        """"""
        data = DepartmentActive(date="2024-10-28").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_department_rank(self):
        """"""
        data = DepartmentRank(statistics_cycle="1m").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_department_statistic(self):
        """"""
        data = DepartmentStatistic(statistics_cycle="1m").load()
        print(data.to_frame(chinese_column=True).to_markdown())



__all__ = [
    "headers",
    "trading_technicals_mapping",
]

headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36',
}

trading_technicals_mapping = {
    "1min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C1%2CRecommend.All%7C1%2CRecommend.MA%7C1%2CRSI%7C1%2CRSI%5B1%5D%7C1%2CStoch.K%7C1%2CStoch.D%7C1%2CStoch.K%5B1%5D%7C1%2CStoch.D%5B1%5D%7C1%2CCCI20%7C1%2CCCI20%5B1%5D%7C1%2CADX%7C1%2CADX%2BDI%7C1%2CADX-DI%7C1%2CADX%2BDI%5B1%5D%7C1%2CADX-DI%5B1%5D%7C1%2CAO%7C1%2CAO%5B1%5D%7C1%2CAO%5B2%5D%7C1%2CMom%7C1%2CMom%5B1%5D%7C1%2CMACD.macd%7C1%2CMACD.signal%7C1%2CRec.Stoch.RSI%7C1%2CStoch.RSI.K%7C1%2CRec.WR%7C1%2CW.R%7C1%2CRec.BBPower%7C1%2CBBPower%7C1%2CRec.UO%7C1%2CUO%7C1%2CEMA10%7C1%2Cclose%7C1%2CSMA10%7C1%2CEMA20%7C1%2CSMA20%7C1%2CEMA30%7C1%2CSMA30%7C1%2CEMA50%7C1%2CSMA50%7C1%2CEMA100%7C1%2CSMA100%7C1%2CEMA200%7C1%2CSMA200%7C1%2CRec.Ichimoku%7C1%2CIchimoku.BLine%7C1%2CRec.VWMA%7C1%2CVWMA%7C1%2CRec.HullMA9%7C1%2CHullMA9%7C1%2CPivot.M.Classic.S3%7C1%2CPivot.M.Classic.S2%7C1%2CPivot.M.Classic.S1%7C1%2CPivot.M.Classic.Middle%7C1%2CPivot.M.Classic.R1%7C1%2CPivot.M.Classic.R2%7C1%2CPivot.M.Classic.R3%7C1%2CPivot.M.Fibonacci.S3%7C1%2CPivot.M.Fibonacci.S2%7C1%2CPivot.M.Fibonacci.S1%7C1%2CPivot.M.Fibonacci.Middle%7C1%2CPivot.M.Fibonacci.R1%7C1%2CPivot.M.Fibonacci.R2%7C1%2CPivot.M.Fibonacci.R3%7C1%2CPivot.M.Camarilla.S3%7C1%2CPivot.M.Camarilla.S2%7C1%2CPivot.M.Camarilla.S1%7C1%2CPivot.M.Camarilla.Middle%7C1%2CPivot.M.Camarilla.R1%7C1%2CPivot.M.Camarilla.R2%7C1%2CPivot.M.Camarilla.R3%7C1%2CPivot.M.Woodie.S3%7C1%2CPivot.M.Woodie.S2%7C1%2CPivot.M.Woodie.S1%7C1%2CPivot.M.Woodie.Middle%7C1%2CPivot.M.Woodie.R1%7C1%2CPivot.M.Woodie.R2%7C1%2CPivot.M.Woodie.R3%7C1%2CPivot.M.Demark.S1%7C1%2CPivot.M.Demark.Middle%7C1%2CPivot.M.Demark.R1%7C1&no_404=true",
    "5min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C5%2CRecommend.All%7C5%2CRecommend.MA%7C5%2CRSI%7C5%2CRSI%5B1%5D%7C5%2CStoch.K%7C5%2CStoch.D%7C5%2CStoch.K%5B1%5D%7C5%2CStoch.D%5B1%5D%7C5%2CCCI20%7C5%2CCCI20%5B1%5D%7C5%2CADX%7C5%2CADX%2BDI%7C5%2CADX-DI%7C5%2CADX%2BDI%5B1%5D%7C5%2CADX-DI%5B1%5D%7C5%2CAO%7C5%2CAO%5B1%5D%7C5%2CAO%5B2%5D%7C5%2CMom%7C5%2CMom%5B1%5D%7C5%2CMACD.macd%7C5%2CMACD.signal%7C5%2CRec.Stoch.RSI%7C5%2CStoch.RSI.K%7C5%2CRec.WR%7C5%2CW.R%7C5%2CRec.BBPower%7C5%2CBBPower%7C5%2CRec.UO%7C5%2CUO%7C5%2CEMA10%7C5%2Cclose%7C5%2CSMA10%7C5%2CEMA20%7C5%2CSMA20%7C5%2CEMA30%7C5%2CSMA30%7C5%2CEMA50%7C5%2CSMA50%7C5%2CEMA100%7C5%2CSMA100%7C5%2CEMA200%7C5%2CSMA200%7C5%2CRec.Ichimoku%7C5%2CIchimoku.BLine%7C5%2CRec.VWMA%7C5%2CVWMA%7C5%2CRec.HullMA9%7C5%2CHullMA9%7C5%2CPivot.M.Classic.S3%7C5%2CPivot.M.Classic.S2%7C5%2CPivot.M.Classic.S1%7C5%2CPivot.M.Classic.Middle%7C5%2CPivot.M.Classic.R1%7C5%2CPivot.M.Classic.R2%7C5%2CPivot.M.Classic.R3%7C5%2CPivot.M.Fibonacci.S3%7C5%2CPivot.M.Fibonacci.S2%7C5%2CPivot.M.Fibonacci.S1%7C5%2CPivot.M.Fibonacci.Middle%7C5%2CPivot.M.Fibonacci.R1%7C5%2CPivot.M.Fibonacci.R2%7C5%2CPivot.M.Fibonacci.R3%7C5%2CPivot.M.Camarilla.S3%7C5%2CPivot.M.Camarilla.S2%7C5%2CPivot.M.Camarilla.S1%7C5%2CPivot.M.Camarilla.Middle%7C5%2CPivot.M.Camarilla.R1%7C5%2CPivot.M.Camarilla.R2%7C5%2CPivot.M.Camarilla.R3%7C5%2CPivot.M.Woodie.S3%7C5%2CPivot.M.Woodie.S2%7C5%2CPivot.M.Woodie.S1%7C5%2CPivot.M.Woodie.Middle%7C5%2CPivot.M.Woodie.R1%7C5%2CPivot.M.Woodie.R2%7C5%2CPivot.M.Woodie.R3%7C5%2CPivot.M.Demark.S1%7C5%2CPivot.M.Demark.Middle%7C5%2CPivot.M.Demark.R1%7C5&no_404=true",
    "15min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C15%2CRecommend.All%7C15%2CRecommend.MA%7C15%2CRSI%7C15%2CRSI%5B1%5D%7C15%2CStoch.K%7C15%2CStoch.D%7C15%2CStoch.K%5B1%5D%7C15%2CStoch.D%5B1%5D%7C15%2CCCI20%7C15%2CCCI20%5B1%5D%7C15%2CADX%7C15%2CADX%2BDI%7C15%2CADX-DI%7C15%2CADX%2BDI%5B1%5D%7C15%2CADX-DI%5B1%5D%7C15%2CAO%7C15%2CAO%5B1%5D%7C15%2CAO%5B2%5D%7C15%2CMom%7C15%2CMom%5B1%5D%7C15%2CMACD.macd%7C15%2CMACD.signal%7C15%2CRec.Stoch.RSI%7C15%2CStoch.RSI.K%7C15%2CRec.WR%7C15%2CW.R%7C15%2CRec.BBPower%7C15%2CBBPower%7C15%2CRec.UO%7C15%2CUO%7C15%2CEMA10%7C15%2Cclose%7C15%2CSMA10%7C15%2CEMA20%7C15%2CSMA20%7C15%2CEMA30%7C15%2CSMA30%7C15%2CEMA50%7C15%2CSMA50%7C15%2CEMA100%7C15%2CSMA100%7C15%2CEMA200%7C15%2CSMA200%7C15%2CRec.Ichimoku%7C15%2CIchimoku.BLine%7C15%2CRec.VWMA%7C15%2CVWMA%7C15%2CRec.HullMA9%7C15%2CHullMA9%7C15%2CPivot.M.Classic.S3%7C15%2CPivot.M.Classic.S2%7C15%2CPivot.M.Classic.S1%7C15%2CPivot.M.Classic.Middle%7C15%2CPivot.M.Classic.R1%7C15%2CPivot.M.Classic.R2%7C15%2CPivot.M.Classic.R3%7C15%2CPivot.M.Fibonacci.S3%7C15%2CPivot.M.Fibonacci.S2%7C15%2CPivot.M.Fibonacci.S1%7C15%2CPivot.M.Fibonacci.Middle%7C15%2CPivot.M.Fibonacci.R1%7C15%2CPivot.M.Fibonacci.R2%7C15%2CPivot.M.Fibonacci.R3%7C15%2CPivot.M.Camarilla.S3%7C15%2CPivot.M.Camarilla.S2%7C15%2CPivot.M.Camarilla.S1%7C15%2CPivot.M.Camarilla.Middle%7C15%2CPivot.M.Camarilla.R1%7C15%2CPivot.M.Camarilla.R2%7C15%2CPivot.M.Camarilla.R3%7C15%2CPivot.M.Woodie.S3%7C15%2CPivot.M.Woodie.S2%7C15%2CPivot.M.Woodie.S1%7C15%2CPivot.M.Woodie.Middle%7C15%2CPivot.M.Woodie.R1%7C15%2CPivot.M.Woodie.R2%7C15%2CPivot.M.Woodie.R3%7C15%2CPivot.M.Demark.S1%7C15%2CPivot.M.Demark.Middle%7C15%2CPivot.M.Demark.R1%7C15&no_404=true".format,
    "30min:": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C30%2CRecommend.All%7C30%2CRecommend.MA%7C30%2CRSI%7C30%2CRSI%5B1%5D%7C30%2CStoch.K%7C30%2CStoch.D%7C30%2CStoch.K%5B1%5D%7C30%2CStoch.D%5B1%5D%7C30%2CCCI20%7C30%2CCCI20%5B1%5D%7C30%2CADX%7C30%2CADX%2BDI%7C30%2CADX-DI%7C30%2CADX%2BDI%5B1%5D%7C30%2CADX-DI%5B1%5D%7C30%2CAO%7C30%2CAO%5B1%5D%7C30%2CAO%5B2%5D%7C30%2CMom%7C30%2CMom%5B1%5D%7C30%2CMACD.macd%7C30%2CMACD.signal%7C30%2CRec.Stoch.RSI%7C30%2CStoch.RSI.K%7C30%2CRec.WR%7C30%2CW.R%7C30%2CRec.BBPower%7C30%2CBBPower%7C30%2CRec.UO%7C30%2CUO%7C30%2CEMA10%7C30%2Cclose%7C30%2CSMA10%7C30%2CEMA20%7C30%2CSMA20%7C30%2CEMA30%7C30%2CSMA30%7C30%2CEMA50%7C30%2CSMA50%7C30%2CEMA100%7C30%2CSMA100%7C30%2CEMA200%7C30%2CSMA200%7C30%2CRec.Ichimoku%7C30%2CIchimoku.BLine%7C30%2CRec.VWMA%7C30%2CVWMA%7C30%2CRec.HullMA9%7C30%2CHullMA9%7C30%2CPivot.M.Classic.S3%7C30%2CPivot.M.Classic.S2%7C30%2CPivot.M.Classic.S1%7C30%2CPivot.M.Classic.Middle%7C30%2CPivot.M.Classic.R1%7C30%2CPivot.M.Classic.R2%7C30%2CPivot.M.Classic.R3%7C30%2CPivot.M.Fibonacci.S3%7C30%2CPivot.M.Fibonacci.S2%7C30%2CPivot.M.Fibonacci.S1%7C30%2CPivot.M.Fibonacci.Middle%7C30%2CPivot.M.Fibonacci.R1%7C30%2CPivot.M.Fibonacci.R2%7C30%2CPivot.M.Fibonacci.R3%7C30%2CPivot.M.Camarilla.S3%7C30%2CPivot.M.Camarilla.S2%7C30%2CPivot.M.Camarilla.S1%7C30%2CPivot.M.Camarilla.Middle%7C30%2CPivot.M.Camarilla.R1%7C30%2CPivot.M.Camarilla.R2%7C30%2CPivot.M.Camarilla.R3%7C30%2CPivot.M.Woodie.S3%7C30%2CPivot.M.Woodie.S2%7C30%2CPivot.M.Woodie.S1%7C30%2CPivot.M.Woodie.Middle%7C30%2CPivot.M.Woodie.R1%7C30%2CPivot.M.Woodie.R2%7C30%2CPivot.M.Woodie.R3%7C30%2CPivot.M.Demark.S1%7C30%2CPivot.M.Demark.Middle%7C30%2CPivot.M.Demark.R1%7C30&no_404=true".format,
    "60min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C60%2CRecommend.All%7C60%2CRecommend.MA%7C60%2CRSI%7C60%2CRSI%5B1%5D%7C60%2CStoch.K%7C60%2CStoch.D%7C60%2CStoch.K%5B1%5D%7C60%2CStoch.D%5B1%5D%7C60%2CCCI20%7C60%2CCCI20%5B1%5D%7C60%2CADX%7C60%2CADX%2BDI%7C60%2CADX-DI%7C60%2CADX%2BDI%5B1%5D%7C60%2CADX-DI%5B1%5D%7C60%2CAO%7C60%2CAO%5B1%5D%7C60%2CAO%5B2%5D%7C60%2CMom%7C60%2CMom%5B1%5D%7C60%2CMACD.macd%7C60%2CMACD.signal%7C60%2CRec.Stoch.RSI%7C60%2CStoch.RSI.K%7C60%2CRec.WR%7C60%2CW.R%7C60%2CRec.BBPower%7C60%2CBBPower%7C60%2CRec.UO%7C60%2CUO%7C60%2CEMA10%7C60%2Cclose%7C60%2CSMA10%7C60%2CEMA20%7C60%2CSMA20%7C60%2CEMA30%7C60%2CSMA30%7C60%2CEMA50%7C60%2CSMA50%7C60%2CEMA100%7C60%2CSMA100%7C60%2CEMA200%7C60%2CSMA200%7C60%2CRec.Ichimoku%7C60%2CIchimoku.BLine%7C60%2CRec.VWMA%7C60%2CVWMA%7C60%2CRec.HullMA9%7C60%2CHullMA9%7C60%2CPivot.M.Classic.S3%7C60%2CPivot.M.Classic.S2%7C60%2CPivot.M.Classic.S1%7C60%2CPivot.M.Classic.Middle%7C60%2CPivot.M.Classic.R1%7C60%2CPivot.M.Classic.R2%7C60%2CPivot.M.Classic.R3%7C60%2CPivot.M.Fibonacci.S3%7C60%2CPivot.M.Fibonacci.S2%7C60%2CPivot.M.Fibonacci.S1%7C60%2CPivot.M.Fibonacci.Middle%7C60%2CPivot.M.Fibonacci.R1%7C60%2CPivot.M.Fibonacci.R2%7C60%2CPivot.M.Fibonacci.R3%7C60%2CPivot.M.Camarilla.S3%7C60%2CPivot.M.Camarilla.S2%7C60%2CPivot.M.Camarilla.S1%7C60%2CPivot.M.Camarilla.Middle%7C60%2CPivot.M.Camarilla.R1%7C60%2CPivot.M.Camarilla.R2%7C60%2CPivot.M.Camarilla.R3%7C60%2CPivot.M.Woodie.S3%7C60%2CPivot.M.Woodie.S2%7C60%2CPivot.M.Woodie.S1%7C60%2CPivot.M.Woodie.Middle%7C60%2CPivot.M.Woodie.R1%7C60%2CPivot.M.Woodie.R2%7C60%2CPivot.M.Woodie.R3%7C60%2CPivot.M.Demark.S1%7C60%2CPivot.M.Demark.Middle%7C60%2CPivot.M.Demark.R1%7C60&no_404=true".format,
    "120min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C120%2CRecommend.All%7C120%2CRecommend.MA%7C120%2CRSI%7C120%2CRSI%5B1%5D%7C120%2CStoch.K%7C120%2CStoch.D%7C120%2CStoch.K%5B1%5D%7C120%2CStoch.D%5B1%5D%7C120%2CCCI20%7C120%2CCCI20%5B1%5D%7C120%2CADX%7C120%2CADX%2BDI%7C120%2CADX-DI%7C120%2CADX%2BDI%5B1%5D%7C120%2CADX-DI%5B1%5D%7C120%2CAO%7C120%2CAO%5B1%5D%7C120%2CAO%5B2%5D%7C120%2CMom%7C120%2CMom%5B1%5D%7C120%2CMACD.macd%7C120%2CMACD.signal%7C120%2CRec.Stoch.RSI%7C120%2CStoch.RSI.K%7C120%2CRec.WR%7C120%2CW.R%7C120%2CRec.BBPower%7C120%2CBBPower%7C120%2CRec.UO%7C120%2CUO%7C120%2CEMA10%7C120%2Cclose%7C120%2CSMA10%7C120%2CEMA20%7C120%2CSMA20%7C120%2CEMA30%7C120%2CSMA30%7C120%2CEMA50%7C120%2CSMA50%7C120%2CEMA100%7C120%2CSMA100%7C120%2CEMA200%7C120%2CSMA200%7C120%2CRec.Ichimoku%7C120%2CIchimoku.BLine%7C120%2CRec.VWMA%7C120%2CVWMA%7C120%2CRec.HullMA9%7C120%2CHullMA9%7C120%2CPivot.M.Classic.S3%7C120%2CPivot.M.Classic.S2%7C120%2CPivot.M.Classic.S1%7C120%2CPivot.M.Classic.Middle%7C120%2CPivot.M.Classic.R1%7C120%2CPivot.M.Classic.R2%7C120%2CPivot.M.Classic.R3%7C120%2CPivot.M.Fibonacci.S3%7C120%2CPivot.M.Fibonacci.S2%7C120%2CPivot.M.Fibonacci.S1%7C120%2CPivot.M.Fibonacci.Middle%7C120%2CPivot.M.Fibonacci.R1%7C120%2CPivot.M.Fibonacci.R2%7C120%2CPivot.M.Fibonacci.R3%7C120%2CPivot.M.Camarilla.S3%7C120%2CPivot.M.Camarilla.S2%7C120%2CPivot.M.Camarilla.S1%7C120%2CPivot.M.Camarilla.Middle%7C120%2CPivot.M.Camarilla.R1%7C120%2CPivot.M.Camarilla.R2%7C120%2CPivot.M.Camarilla.R3%7C120%2CPivot.M.Woodie.S3%7C120%2CPivot.M.Woodie.S2%7C120%2CPivot.M.Woodie.S1%7C120%2CPivot.M.Woodie.Middle%7C120%2CPivot.M.Woodie.R1%7C120%2CPivot.M.Woodie.R2%7C120%2CPivot.M.Woodie.R3%7C120%2CPivot.M.Demark.S1%7C120%2CPivot.M.Demark.Middle%7C120%2CPivot.M.Demark.R1%7C120&no_404=true".format,
    "240min": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C240%2CRecommend.All%7C240%2CRecommend.MA%7C240%2CRSI%7C240%2CRSI%5B1%5D%7C240%2CStoch.K%7C240%2CStoch.D%7C240%2CStoch.K%5B1%5D%7C240%2CStoch.D%5B1%5D%7C240%2CCCI20%7C240%2CCCI20%5B1%5D%7C240%2CADX%7C240%2CADX%2BDI%7C240%2CADX-DI%7C240%2CADX%2BDI%5B1%5D%7C240%2CADX-DI%5B1%5D%7C240%2CAO%7C240%2CAO%5B1%5D%7C240%2CAO%5B2%5D%7C240%2CMom%7C240%2CMom%5B1%5D%7C240%2CMACD.macd%7C240%2CMACD.signal%7C240%2CRec.Stoch.RSI%7C240%2CStoch.RSI.K%7C240%2CRec.WR%7C240%2CW.R%7C240%2CRec.BBPower%7C240%2CBBPower%7C240%2CRec.UO%7C240%2CUO%7C240%2CEMA10%7C240%2Cclose%7C240%2CSMA10%7C240%2CEMA20%7C240%2CSMA20%7C240%2CEMA30%7C240%2CSMA30%7C240%2CEMA50%7C240%2CSMA50%7C240%2CEMA100%7C240%2CSMA100%7C240%2CEMA200%7C240%2CSMA200%7C240%2CRec.Ichimoku%7C240%2CIchimoku.BLine%7C240%2CRec.VWMA%7C240%2CVWMA%7C240%2CRec.HullMA9%7C240%2CHullMA9%7C240%2CPivot.M.Classic.S3%7C240%2CPivot.M.Classic.S2%7C240%2CPivot.M.Classic.S1%7C240%2CPivot.M.Classic.Middle%7C240%2CPivot.M.Classic.R1%7C240%2CPivot.M.Classic.R2%7C240%2CPivot.M.Classic.R3%7C240%2CPivot.M.Fibonacci.S3%7C240%2CPivot.M.Fibonacci.S2%7C240%2CPivot.M.Fibonacci.S1%7C240%2CPivot.M.Fibonacci.Middle%7C240%2CPivot.M.Fibonacci.R1%7C240%2CPivot.M.Fibonacci.R2%7C240%2CPivot.M.Fibonacci.R3%7C240%2CPivot.M.Camarilla.S3%7C240%2CPivot.M.Camarilla.S2%7C240%2CPivot.M.Camarilla.S1%7C240%2CPivot.M.Camarilla.Middle%7C240%2CPivot.M.Camarilla.R1%7C240%2CPivot.M.Camarilla.R2%7C240%2CPivot.M.Camarilla.R3%7C240%2CPivot.M.Woodie.S3%7C240%2CPivot.M.Woodie.S2%7C240%2CPivot.M.Woodie.S1%7C240%2CPivot.M.Woodie.Middle%7C240%2CPivot.M.Woodie.R1%7C240%2CPivot.M.Woodie.R2%7C240%2CPivot.M.Woodie.R3%7C240%2CPivot.M.Demark.S1%7C240%2CPivot.M.Demark.Middle%7C240%2CPivot.M.Demark.R1%7C240&no_404=true".format,
    "1day": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%2CRecommend.All%2CRecommend.MA%2CRSI%2CRSI%5B1%5D%2CStoch.K%2CStoch.D%2CStoch.K%5B1%5D%2CStoch.D%5B1%5D%2CCCI20%2CCCI20%5B1%5D%2CADX%2CADX%2BDI%2CADX-DI%2CADX%2BDI%5B1%5D%2CADX-DI%5B1%5D%2CAO%2CAO%5B1%5D%2CAO%5B2%5D%2CMom%2CMom%5B1%5D%2CMACD.macd%2CMACD.signal%2CRec.Stoch.RSI%2CStoch.RSI.K%2CRec.WR%2CW.R%2CRec.BBPower%2CBBPower%2CRec.UO%2CUO%2CEMA10%2Cclose%2CSMA10%2CEMA20%2CSMA20%2CEMA30%2CSMA30%2CEMA50%2CSMA50%2CEMA100%2CSMA100%2CEMA200%2CSMA200%2CRec.Ichimoku%2CIchimoku.BLine%2CRec.VWMA%2CVWMA%2CRec.HullMA9%2CHullMA9%2CPivot.M.Classic.S3%2CPivot.M.Classic.S2%2CPivot.M.Classic.S1%2CPivot.M.Classic.Middle%2CPivot.M.Classic.R1%2CPivot.M.Classic.R2%2CPivot.M.Classic.R3%2CPivot.M.Fibonacci.S3%2CPivot.M.Fibonacci.S2%2CPivot.M.Fibonacci.S1%2CPivot.M.Fibonacci.Middle%2CPivot.M.Fibonacci.R1%2CPivot.M.Fibonacci.R2%2CPivot.M.Fibonacci.R3%2CPivot.M.Camarilla.S3%2CPivot.M.Camarilla.S2%2CPivot.M.Camarilla.S1%2CPivot.M.Camarilla.Middle%2CPivot.M.Camarilla.R1%2CPivot.M.Camarilla.R2%2CPivot.M.Camarilla.R3%2CPivot.M.Woodie.S3%2CPivot.M.Woodie.S2%2CPivot.M.Woodie.S1%2CPivot.M.Woodie.Middle%2CPivot.M.Woodie.R1%2CPivot.M.Woodie.R2%2CPivot.M.Woodie.R3%2CPivot.M.Demark.S1%2CPivot.M.Demark.Middle%2CPivot.M.Demark.R1&no_404=true".format,
    "1week": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C1W%2CRecommend.All%7C1W%2CRecommend.MA%7C1W%2CRSI%7C1W%2CRSI%5B1%5D%7C1W%2CStoch.K%7C1W%2CStoch.D%7C1W%2CStoch.K%5B1%5D%7C1W%2CStoch.D%5B1%5D%7C1W%2CCCI20%7C1W%2CCCI20%5B1%5D%7C1W%2CADX%7C1W%2CADX%2BDI%7C1W%2CADX-DI%7C1W%2CADX%2BDI%5B1%5D%7C1W%2CADX-DI%5B1%5D%7C1W%2CAO%7C1W%2CAO%5B1%5D%7C1W%2CAO%5B2%5D%7C1W%2CMom%7C1W%2CMom%5B1%5D%7C1W%2CMACD.macd%7C1W%2CMACD.signal%7C1W%2CRec.Stoch.RSI%7C1W%2CStoch.RSI.K%7C1W%2CRec.WR%7C1W%2CW.R%7C1W%2CRec.BBPower%7C1W%2CBBPower%7C1W%2CRec.UO%7C1W%2CUO%7C1W%2CEMA10%7C1W%2Cclose%7C1W%2CSMA10%7C1W%2CEMA20%7C1W%2CSMA20%7C1W%2CEMA30%7C1W%2CSMA30%7C1W%2CEMA50%7C1W%2CSMA50%7C1W%2CEMA100%7C1W%2CSMA100%7C1W%2CEMA200%7C1W%2CSMA200%7C1W%2CRec.Ichimoku%7C1W%2CIchimoku.BLine%7C1W%2CRec.VWMA%7C1W%2CVWMA%7C1W%2CRec.HullMA9%7C1W%2CHullMA9%7C1W%2CPivot.M.Classic.S3%7C1W%2CPivot.M.Classic.S2%7C1W%2CPivot.M.Classic.S1%7C1W%2CPivot.M.Classic.Middle%7C1W%2CPivot.M.Classic.R1%7C1W%2CPivot.M.Classic.R2%7C1W%2CPivot.M.Classic.R3%7C1W%2CPivot.M.Fibonacci.S3%7C1W%2CPivot.M.Fibonacci.S2%7C1W%2CPivot.M.Fibonacci.S1%7C1W%2CPivot.M.Fibonacci.Middle%7C1W%2CPivot.M.Fibonacci.R1%7C1W%2CPivot.M.Fibonacci.R2%7C1W%2CPivot.M.Fibonacci.R3%7C1W%2CPivot.M.Camarilla.S3%7C1W%2CPivot.M.Camarilla.S2%7C1W%2CPivot.M.Camarilla.S1%7C1W%2CPivot.M.Camarilla.Middle%7C1W%2CPivot.M.Camarilla.R1%7C1W%2CPivot.M.Camarilla.R2%7C1W%2CPivot.M.Camarilla.R3%7C1W%2CPivot.M.Woodie.S3%7C1W%2CPivot.M.Woodie.S2%7C1W%2CPivot.M.Woodie.S1%7C1W%2CPivot.M.Woodie.Middle%7C1W%2CPivot.M.Woodie.R1%7C1W%2CPivot.M.Woodie.R2%7C1W%2CPivot.M.Woodie.R3%7C1W%2CPivot.M.Demark.S1%7C1W%2CPivot.M.Demark.Middle%7C1W%2CPivot.M.Demark.R1%7C1W&no_404=true".format,
    "1month": "https://scanner.tradingview.com/symbol?symbol={}%3A{}&fields=Recommend.Other%7C1M%2CRecommend.All%7C1M%2CRecommend.MA%7C1M%2CRSI%7C1M%2CRSI%5B1%5D%7C1M%2CStoch.K%7C1M%2CStoch.D%7C1M%2CStoch.K%5B1%5D%7C1M%2CStoch.D%5B1%5D%7C1M%2CCCI20%7C1M%2CCCI20%5B1%5D%7C1M%2CADX%7C1M%2CADX%2BDI%7C1M%2CADX-DI%7C1M%2CADX%2BDI%5B1%5D%7C1M%2CADX-DI%5B1%5D%7C1M%2CAO%7C1M%2CAO%5B1%5D%7C1M%2CAO%5B2%5D%7C1M%2CMom%7C1M%2CMom%5B1%5D%7C1M%2CMACD.macd%7C1M%2CMACD.signal%7C1M%2CRec.Stoch.RSI%7C1M%2CStoch.RSI.K%7C1M%2CRec.WR%7C1M%2CW.R%7C1M%2CRec.BBPower%7C1M%2CBBPower%7C1M%2CRec.UO%7C1M%2CUO%7C1M%2CEMA10%7C1M%2Cclose%7C1M%2CSMA10%7C1M%2CEMA20%7C1M%2CSMA20%7C1M%2CEMA30%7C1M%2CSMA30%7C1M%2CEMA50%7C1M%2CSMA50%7C1M%2CEMA100%7C1M%2CSMA100%7C1M%2CEMA200%7C1M%2CSMA200%7C1M%2CRec.Ichimoku%7C1M%2CIchimoku.BLine%7C1M%2CRec.VWMA%7C1M%2CVWMA%7C1M%2CRec.HullMA9%7C1M%2CHullMA9%7C1M%2CPivot.M.Classic.S3%7C1M%2CPivot.M.Classic.S2%7C1M%2CPivot.M.Classic.S1%7C1M%2CPivot.M.Classic.Middle%7C1M%2CPivot.M.Classic.R1%7C1M%2CPivot.M.Classic.R2%7C1M%2CPivot.M.Classic.R3%7C1M%2CPivot.M.Fibonacci.S3%7C1M%2CPivot.M.Fibonacci.S2%7C1M%2CPivot.M.Fibonacci.S1%7C1M%2CPivot.M.Fibonacci.Middle%7C1M%2CPivot.M.Fibonacci.R1%7C1M%2CPivot.M.Fibonacci.R2%7C1M%2CPivot.M.Fibonacci.R3%7C1M%2CPivot.M.Camarilla.S3%7C1M%2CPivot.M.Camarilla.S2%7C1M%2CPivot.M.Camarilla.S1%7C1M%2CPivot.M.Camarilla.Middle%7C1M%2CPivot.M.Camarilla.R1%7C1M%2CPivot.M.Camarilla.R2%7C1M%2CPivot.M.Camarilla.R3%7C1M%2CPivot.M.Woodie.S3%7C1M%2CPivot.M.Woodie.S2%7C1M%2CPivot.M.Woodie.S1%7C1M%2CPivot.M.Woodie.Middle%7C1M%2CPivot.M.Woodie.R1%7C1M%2CPivot.M.Woodie.R2%7C1M%2CPivot.M.Woodie.R3%7C1M%2CPivot.M.Demark.S1%7C1M%2CPivot.M.Demark.Middle%7C1M%2CPivot.M.Demark.R1%7C1M&no_404=true".format,
}



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APICarHistoryRequestData


__all__ = [
    "CarHistorySalesMapping",
    "CarHistorySales",
]


class CarHistorySalesMapping(BaseMapping):
    """字段映射 汽车历史销量"""
    columns: Dict = {}


class CarHistorySales(APICarHistoryRequestData):
    """查询 汽车历史销量"""
    def __init__(
            self,
            data_type: Literal["汽车销量"] = "汽车销量",
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            data = CarHistorySales(data_type="电动车销量",).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            data_type: 查询类型
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.data_type = data_type
        self.mapping = CarHistorySalesMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="汽车历史销量", response_type="html")
        self.data_mark = self.params_data_mark()
        self.params_multi_pages()

    def params_data_mark(self) -> str:
        """"""
        data_mark_mapping = {
            "汽车销量": "month",
            "品牌销量": "brand",
        }
        return data_mark_mapping[self.data_type]

    def params_multi_pages(self):
        if self.data_type in ["汽车销量", "品牌销量"]:
            self.multi_pages = True
        else:
            self.multi_pages = False



from typing import Optional, List, Dict, Any
from wisecon.types import ResponseData

from .asset_invest import *
from .boom_index import *
from .cpi import *
from .currency_supply import *
from .customs import *
from .deposit_rate import *
from .deposit_reserve import *
from .faith_index import *
from .fdi import *
from .forex_deposit import *
from .forex_loan import *
from .gdp import *
from .gold_currency import *
from .goodes_index import *
from .gov_income import *
from .hose_index import *
from .imp_intrestraten import *
from .indus_grow import *
from .lpr import *
from .new_loan import *
from .oil_price import *
from .pmi import *
from .ppi import *
from .stock_open import *
from .stock_statistics import *
from .tax import *
from .total_retail import *
from .transfer_fund import *


__all__ = [
    "MacroIndex"
]


data_mapping: Dict = {
        "居民消费价格指数(CPI)": CPI,
        "工业品出厂价格指数(PPI)": PPI,
        "国内生产总值(GDP)": GDP,
        "采购经理人指数(PMI)": PMI,
        "城镇固定资产投资": AssetInvest,
        "房价指数(old)": HoseIndexOld,
        "新房价指数": HoseIndexNew,
        "企业景气及企业家信心指数": BoomIndex,
        "工业增加值增长": IndusGrow,
        "企业商品价格指数": GoodsIndex,
        "消费者信心指数": FaithIndex,
        "社会消费品零售总额": TotalRetail,
        "货币供应量": CurrencySupply,
        "海关进出口增减情况一览表": Customs,
        "全国股票交易统计表": StockStatistics,
        "外汇和黄金储备": GoldCurrency,
        "交易结算资金(银证转账)": TransferFund,
        "股票账户统计表(新)": StockOpen,
        "外商直接投资数据(FDI)": FDI,
        "财政收入": GovIncome,
        "全国税收收入": Tax,
        "新增信贷数据": NewLoan,
        "银行间拆借利率": ImpInterest,
        "本外币存款": ForexDeposit,
        "外汇贷款数据": ForexLoan,
        "存款准备金率": DepositReserve,
        "利率调整": DepositRate,
        "油价": OilPrice,
        "LPR数据": LPR,
    }


class MacroIndex:
    def __init__(self):
        """"""

    def list_data(self) -> List[str]:
        """"""
        return list(data_mapping.keys())

    def load_data(
            self,
            macro: str,
            size: Optional[int] = 20,
            **kwargs: Any,
    ) -> ResponseData:
        """"""
        if macro not in data_mapping:
            raise ValueError(f"Invalid macro index: {macro}, valid options: {list(data_mapping.keys())}")

        new_loan = data_mapping.get(macro)(size=size, **kwargs)
        data = new_loan.load_data()
        return data



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "FreeHolder",
    "FreeHolderMapping",
]


class FreeHolderMapping(BaseMapping):
    """字段映射 上市公司十大流通股东持股明细"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "ORG_CODE": "机构代码",
        "END_DATE": "截止日期",
        "HOLDER_NAME": "股东名称",
        "HOLD_NUM": "持有股数",
        "FREE_HOLDNUM_RATIO": "自由持股比例",
        "HOLD_NUM_CHANGE": "持股变动",
        "CHANGE_RATIO": "变动比例",
        "IS_HOLDORG": "是否持股机构",
        "HOLDER_RANK": "股东排名",
        "SECURITY_NAME_ABBR": "证券简称",
        "HOLDER_CODE": "股东代码",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "HOLDER_STATE": "股东状态",
        "HOLDER_MARKET_CAP": "股东市值",
        "HOLD_RATIO": "持股比例",
        "HOLD_CHANGE": "持股变动",
        "HOLD_RATIO_CHANGE": "持股比例变动",
        "HOLDER_TYPE": "股东类型",
        "SHARES_TYPE": "股份类型",
        "UPDATE_DATE": "更新时间",
        "REPORT_DATE_NAME": "报告名称",
        "HOLDER_NEW": "新的股东",
        "FREE_RATIO_QOQ": "自由持股比例环比",
        "HOLDER_STATEE": "股东状态",
        "IS_REPORT": "是否报告",
        "HOLDER_CODE_OLD": "旧股东代码",
        "HOLDER_NEWTYPE": "新股东类型",
        "HOLDNUM_CHANGE_NAME": "持股变动名称",
        "IS_MAX_REPORTDATE": "是否为最新报告日期",
        "COOPERATION_HOLDER_MARK": "合作股东标识",
        "MXID": "记录ID",
        "LISTING_STATE": "上市状态",
        "XZCHANGE": "新增变动",
        "NEW_CHANGE_RATIO": "新变动比例",
        "HOLDER_STATE_NEW": "新的股东状态",
        "HOLD_ORG_CODE_SOURCE": "持股机构来源代码"
    }


class FreeHolder(APIDataV1RequestData):
    """查询 上市公司十大流通股东持股明细"""
    def __init__(
            self,
            holder_name: Optional[str] = None,
            security_code: Optional[str] = None,
            holder_type: Optional[Literal["个人", "基金", "QFII", "社保", "券商", "信托"]] = None,
            holder_change: Optional[Literal["新进", "增加", "不变", "减少"]] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.holder import *

            data = FreeHolder(size=20, start_date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder_name: 股东名称
            security_code: 股票代码
            holder_type: 股东类型 `["个人", "基金", "QFII", "社保", "券商", "信托"]`
            holder_change: 持股变动 `["新进", "增加", "不变", "减少"]`
            size: 返回条数
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否显示日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.holder_name = holder_name
        self.security_code = security_code
        self.holder_type = holder_type
        self.holder_change = holder_change
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = FreeHolderMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司十大流通股东持股明细")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="END_DATE")
        self.filter_security_code()
        if self.holder_type:
            self.conditions.append(f'(HOLDER_NEWTYPE="{self.holder_type}")')
        if self.holder_name:
            self.conditions.append(f'(HOLDER_NAME+like+"%{self.holder_name}%")')
        if self.holder_change:
            self.conditions.append(f'(HOLDNUM_CHANGE_NAME="{self.holder_change}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "UPDATE_DATE,SECURITY_CODE,HOLDER_RANK",
            "sortTypes": "-1,1,1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_F10_EH_FREEHOLDERS",
        }
        return self.base_param(params)



from typing import Any, Dict, Optional, Callable, Literal
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "NewIPOListMapping",
    "NewIPOList",
]


class NewIPOListMapping(BaseMapping):
    """字段映射 新股申购与中签查询"""
    columns: Dict = {
        "SECURITY_CODE": "证券代码",
        "SECURITY_NAME": "证券名称",
        "TRADE_MARKET_CODE": "交易市场代码",
        "APPLY_CODE": "申购代码",
        "TRADE_MARKET": "交易市场",
        "MARKET_TYPE": "市场类型",
        "ORG_TYPE": "组织类型",
        "ISSUE_NUM": "发行数量",
        "ONLINE_ISSUE_NUM": "网上发行数量",
        "OFFLINE_PLACING_NUM": "网下配售数量",
        "TOP_APPLY_MARKETCAP": "最高申购市值",
        "PREDICT_ONFUND_UPPER": "预测资金上限",
        "ONLINE_APPLY_UPPER": "网上申购上限",
        "PREDICT_ONAPPLY_UPPER": "预测申购上限",
        "ISSUE_PRICE": "发行价格",
        "LATELY_PRICE": "最新价格",
        "CLOSE_PRICE": "收盘价格",
        "APPLY_DATE": "申购日期",
        "BALLOT_NUM_DATE": "摇号次数日期",
        "BALLOT_PAY_DATE": "摇号缴款日期",
        "LISTING_DATE": "上市日期",
        "AFTER_ISSUE_PE": "发行后市盈率",
        "ONLINE_ISSUE_LWR": "网上发行下限",
        "INITIAL_MULTIPLE": "初始倍数",
        "INDUSTRY_PE_NEW": "行业新市盈率",
        "OFFLINE_EP_OBJECT": "网下配售对象",
        "CONTINUOUS_1WORD_NUM": "连续1字数",
        "TOTAL_CHANGE": "总变化",
        "PROFIT": "利润",
        "LIMIT_UP_PRICE": "涨停价格",
        "INFO_CODE": "信息代码",
        "OPEN_PRICE": "开盘价格",
        "LD_OPEN_PREMIUM": "开盘溢价",
        "LD_CLOSE_CHANGE": "收盘变动",
        "TURNOVERRATE": "换手率",
        "LD_HIGH_CHANG": "最高变化",
        "LD_AVERAGE_PRICE": "平均价格",
        "OPEN_DATE": "开盘日期",
        "OPEN_AVERAGE_PRICE": "开盘平均价格",
        "PREDICT_PE": "预测市盈率",
        "PREDICT_ISSUE_PRICE2": "预测发行价格2",
        "PREDICT_ISSUE_PRICE": "预测发行价格",
        "PREDICT_ISSUE_PRICE1": "预测发行价格1",
        "PREDICT_ISSUE_PE": "预测发行市盈率",
        "PREDICT_PE_THREE": "预测市盈率三",
        "ONLINE_APPLY_PRICE": "网上申购价格",
        "MAIN_BUSINESS": "主营业务",
        "PAGE_PREDICT_PRICE1": "页面预测价格1",
        "PAGE_PREDICT_PRICE2": "页面预测价格2",
        "PAGE_PREDICT_PRICE3": "页面预测价格3",
        "PAGE_PREDICT_PE1": "页面预测市盈率1",
        "PAGE_PREDICT_PE2": "页面预测市盈率2",
        "PAGE_PREDICT_PE3": "页面预测市盈率3",
        "SELECT_LISTING_DATE": "选择上市日期",
        "IS_BEIJING": "是否北京",
        "INDUSTRY_PE_RATIO": "行业市盈率",
        "INDUSTRY_PE": "行业市盈率",
        "IS_REGISTRATION": "是否注册",
        "IS_REGISTRATION_NEW": "是否新注册",
        "NEWEST_PRICE": "最新价格",

        # RPT_NEEQ_ISSUEINFO_LIST
        "ORG_CODE": "组织代码",
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "EXPECT_ISSUE_NUM": "预计发行数量",
        "PRICE_WAY": "定价方式",
        "ISSUE_PE_RATIO": "发行市盈率",
        "RESULT_NOTICE_DATE": "结果公告日期",
        "APPLY_AMT_UPPER": "申购金额上限",
        "APPLY_NUM_UPPER": "申购股数上限",
        "ONLINE_PAY_DATE": "网上缴款日期",
        "ONLINE_REFUND_DATE": "网上退款日期",
        "PER_SHARES_INCOME": "每股收益",
        "AMPLITUDE": "波动幅度",
        "ONLINE_APPLY_LOWER": "网上申购下限",
        "APPLY_AMT_100": "100股申购金额",
        "TAKE_UP_TIME": "占用时间",
        "CAPTURE_PROFIT": "捕获利润",
        "APPLY_SHARE_100": "100股申购股数",
        "AVERAGE_PRICE": "平均价格",
        "ORG_VAN": "组织价值",
        "VA_AMT": "价值金额",
        "ISSUE_PRICE_ADJFACTOR": "发行价格调整因子",
        "FRIST_CLOSE_CHANGE": "首日收盘变动",
        "f14": "备用字段1",
        "f2": "备用字段2",
        "f3": "备用字段3",
        "NEW_CHANGE_RATE": "新变动率",

        # RPT_BOND_CB_LIST
        "DELIST_DATE": "退市日期",
        "CONVERT_STOCK_CODE": "可转股票代码",
        "BOND_EXPIRE": "到期年限",
        "RATING": "评级",
        "VALUE_DATE": "价值日期",
        "ISSUE_YEAR": "发行年份",
        "CEASE_DATE": "停止交易日期",
        "EXPIRE_DATE": "到期日期",
        "PAY_INTEREST_DAY": "付息日",
        "INTEREST_RATE_EXPLAIN": "利率说明",
        "BOND_COMBINE_CODE": "债券组合代码",
        "ACTUAL_ISSUE_SCALE": "实际发行规模",
        "PAR_VALUE": "面值",
        "ISSUE_OBJECT": "发行对象",
        "REDEEM_TYPE": "赎回类型",
        "EXECUTE_REASON_HS": "执行原因HS",
        "NOTICE_DATE_HS": "通知日期HS",
        "NOTICE_DATE_SH": "通知日期SH",
        "EXECUTE_PRICE_HS": "执行价格HS",
        "EXECUTE_PRICE_SH": "执行价格SH",
        "RECORD_DATE_SH": "登记日期SH",
        "EXECUTE_START_DATESH": "执行开始日期SH",
        "EXECUTE_START_DATEHS": "执行开始日期HS",
        "EXECUTE_END_DATE": "执行结束日期",
        "CORRECODE": "更正代码",
        "CORRECODE_NAME_ABBR": "更正简称",
        "PUBLIC_START_DATE": "公开发行开始日期",
        "CORRECODEO": "更正代码O",
        "CORRECODE_NAME_ABBRO": "更正简称O",
        "BOND_START_DATE": "债券开始日期",
        "SECURITY_START_DATE": "证券开始日期",
        "SECURITY_SHORT_NAME": "证券简称",
        "FIRST_PER_PREPLACING": "首次转让预售",
        "ONLINE_GENERAL_AAU": "网上发行一般额度",
        "ONLINE_GENERAL_LWR": "网上发行下限",
        "INITIAL_TRANSFER_PRICE": "初始转让价格",
        "TRANSFER_END_DATE": "转让结束日期",
        "TRANSFER_START_DATE": "转让开始日期",
        "RESALE_CLAUSE": "转售条款",
        "REDEEM_CLAUSE": "赎回条款",
        "PARTY_NAME": "评级机构名称",
        "CONVERT_STOCK_PRICE": "可转股票价格",
        "TRANSFER_PRICE": "转让价格",
        "TRANSFER_VALUE": "转让价值",
        "CURRENT_BOND_PRICE": "当前债券价格",
        "TRANSFER_PREMIUM_RATIO": "转让溢价比率",
        "CONVERT_STOCK_PRICEHQ": "可转股票价格HQ",
        "MARKET": "市场",
        "RESALE_TRIG_PRICE": "转售触发价格",
        "REDEEM_TRIG_PRICE": "赎回触发价格",
        "PBV_RATIO": "市净率",
        "IB_START_DATE": "IB开始日期",
        "IB_END_DATE": "IB结束日期",
        "CASHFLOW_DATE": "现金流日期",
        "COUPON_IR": "票息利率",
        "PARAM_NAME": "发行参数名称",
        "ISSUE_TYPE": "发行类型",
        "EXECUTE_REASON_SH": "执行原因SH",
        "PAYDAYNEW": "新支付日",
        "CURRENT_BOND_PRICENEW": "当前债券价格新",
        "IS_CONVERT_STOCK": "是否可转股",
        "IS_REDEEM": "是否可赎回",
        "IS_SELLBACK": "是否可回售",
        "FIRST_PROFIT": "首次收益",
        # RPT_CUSTOM_REITS_APPLY_LIST_MARKET
        "SECURITY_INNER_CODE": "证券内部代码",
        "SALE_PRICE": "销售价格",
        "SUBSCRIBE_LOW_NUM": "最低申购金额",
        "ALLOW_SCALE": "允许规模",
        "RAISE_SHARE": "募集份额",
        "RINISTRRATIO": "认购比例",
        "VALID_SUBSCRIBE_MULTIPLE": "有效申购倍数",
        "SUBSCRIBE_START_DATE": "申购开始日期",
        "PUBLIC_VSA_RATIO": "公开发行比例",
        "SUBSCRIBE_END_DATE": "申购结束日期",
        "LTD_SECURITY_NAME": "有限责任公司证券名称",
        "PUBLIC_SHARE": "公开发行股份",
        "SUBSCRIBE_DATE": "申购日期",
        "ENQUIRY_DATE": "询价日期",
        "QUOTA_NONUNIT": "非单位配额",
        "IS_APPLYING": "是否正在申购",
        "IS_LISTING_SHOW": "是否显示上市",
        "PUBLIC_VS_SHARE": "公开发行对比股份",
        "OFFLINE_VS_SHARE": "线下发行对比股份",
        "PUBLIC_VS_MULTIPLE": "公开发行倍数",
        "OFFLINE_VS_MULTIPLE": "线下发行倍数",
        "LD_CHANGE_RATE": "最新变动率",
        "EXPAND_NAME_ABBR": "扩展名称简称",
        "OFFLINE_VSA_RATIO": "线下发行比例",
        "ITEM_TYPE": "项目类型",
        "ITEMTYPE_NAME": "项目类型名称",
        "START_DATE": "开始日期",
        "END_DATE": "结束日期",
        "OPEN": "开盘",
        "CHG": "变动",
        "DTNAVDATE": "日期净值",
        "DTANVPER": "净值周期",
        "DIVIDEND_RATE_TTM": "股息率（TTM）",
        "OPEN_YIELD": "开盘收益",
        "SALE_PRICE_CHG": "销售价格变动",
        "J_PRICE_CHG": "价格变动",
        "PREDICT_RAISE_SCALE": "预测募集规模",
        "NEED_FUND": "所需资金",
        "LD_YIELD": "最新收益",
        "WEEK_START_DATE": "周开始日期",
        "DIVIDEND_RATIO": "股息比率",
        "NEW_DISCOUNT_RATIO": "新折扣率",
        "NEW_DIVIDEND_RATE_TTM": "新股息率（TTM）",
    }


class NewIPOList(APIDataV1RequestData):
    """查询 新股申购与中签查询"""
    def __init__(
            self,
            market: Optional[Literal["全部股票", "沪市主板", "科创板", "深市主板", "创业板", "北交所", "可转债", "REITs"]] = "全部股票",
            size: Optional[int] = 50,
            start_date: Optional[str] = "2010-01-01",
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.new_ipo import *

            data = NewIPOList(market="全部股票").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="沪市主板").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="科创板").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="深市主板").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="创业板").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="北交所").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="可转债").load()
            data.to_frame(chinese_column=True)

            data = NewIPOList(market="REITs").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 股票市场，默认为全部股票， ["沪市主板", "科创板", "深市主板", "创业板", "北交所", "可转债", "REITs"]
            size: 返回数据条数
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否显示详细信息
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.market = market
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = NewIPOListMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="新股申购与中签查询")
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date, date])

    def params_tread_market(self):
        """"""
        market_mapping = {
            "沪市主板": "(SECURITY_TYPE_CODE in (\"058001001\",\"058001008\"))(TRADE_MARKET_CST=\"0101\")",
            "科创板": "(SECURITY_TYPE_CODE in (\"058001001\",\"058001008\"))(TRADE_MARKET_CST=\"0102\")",
            "深市主板": "(SECURITY_TYPE_CODE=\"058001001\")(TRADE_MARKET_CST=\"0201\")",
            "创业板": "(SECURITY_TYPE_CODE=\"058001001\")(TRADE_MARKET_CST=\"0202\")",
        }
        if self.market in market_mapping:
            self.conditions.append(market_mapping[self.market])

    def params_update(self) -> Dict:
        """"""
        if self.market in ["全部股票", "沪市主板", "科创板", "深市主板", "创业板"]:
            return dict()
        elif self.market == "北交所":
            return {
                "columns": "ALL",
                "sortColumns": "APPLY_DATE",
                "sortTypes": "-1",
                "reportName": "RPT_NEEQ_ISSUEINFO_LIST",
                "quoteColumns": "f14,f2~01~SECURITY_CODE,f3~01~SECURITY_CODE,NEW_CHANGE_RATE~01~SECURITY_CODE",
                "quoteType": "0",
                "source": "NEEQSELECT",
            }
        elif self.market == "可转债":
            return {
                "sortColumns": "PUBLIC_START_DATE,SECURITY_CODE",
                "sortTypes": "-1,-1",
                "reportName": "RPT_BOND_CB_LIST",
                "columns": "ALL",
                "quoteColumns": "f2~01~CONVERT_STOCK_CODE~CONVERT_STOCK_PRICE,f235~10~SECURITY_CODE~TRANSFER_PRICE,f236~10~SECURITY_CODE~TRANSFER_VALUE,f2~10~SECURITY_CODE~CURRENT_BOND_PRICE,f237~10~SECURITY_CODE~TRANSFER_PREMIUM_RATIO,f239~10~SECURITY_CODE~RESALE_TRIG_PRICE,f240~10~SECURITY_CODE~REDEEM_TRIG_PRICE,f23~01~CONVERT_STOCK_CODE~PBV_RATIO",
                "quoteType": "0",
            }
        elif self.market == "REITs":
            return {
                "sortColumns": "SUBSCRIBE_START_DATE",
                "sortTypes": "-1",
                "reportName": "RPT_CUSTOM_REITS_APPLY_LIST_MARKET",
                "columns": "ALL",
                "quoteColumns": "f14~09~SECURITY_CODE~LTD_SECURITY_NAME,f2~09~SECURITY_CODE~f2,NEW_DISCOUNT_RATIO~09~SECURITY_CODE,NEW_CHANGE_RATE~09~SECURITY_CODE,NEW_DIVIDEND_RATE_TTM~09~SECURITY_CODE",
            }
        else:
            raise ValueError(f"market: {self.market} is not valid.")

    def params_filter(self) -> str:
        """"""
        if self.market not in ["可转债", "REITs"]:
            self.filter_report_date(date_name="APPLY_DATE")
        self.params_tread_market()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        columns = [
            "SECURITY_CODE", "SECURITY_NAME", "TRADE_MARKET_CODE", "APPLY_CODE", "TRADE_MARKET",
            "MARKET_TYPE", "ORG_TYPE", "ISSUE_NUM", "ONLINE_ISSUE_NUM", "OFFLINE_PLACING_NUM",
            "TOP_APPLY_MARKETCAP", "PREDICT_ONFUND_UPPER", "ONLINE_APPLY_UPPER", "PREDICT_ONAPPLY_UPPER",
            "ISSUE_PRICE", "LATELY_PRICE", "CLOSE_PRICE", "APPLY_DATE", "BALLOT_NUM_DATE",
            "BALLOT_PAY_DATE", "LISTING_DATE", "AFTER_ISSUE_PE", "ONLINE_ISSUE_LWR", "INITIAL_MULTIPLE",
            "INDUSTRY_PE_NEW", "OFFLINE_EP_OBJECT", "CONTINUOUS_1WORD_NUM", "TOTAL_CHANGE", "PROFIT",
            "LIMIT_UP_PRICE", "INFO_CODE", "OPEN_PRICE", "LD_OPEN_PREMIUM", "LD_CLOSE_CHANGE",
            "TURNOVERRATE", "LD_HIGH_CHANG", "LD_AVERAGE_PRICE", "OPEN_DATE", "OPEN_AVERAGE_PRICE",
            "PREDICT_PE", "PREDICT_ISSUE_PRICE2", "PREDICT_ISSUE_PRICE", "PREDICT_ISSUE_PRICE1",
            "PREDICT_ISSUE_PE", "PREDICT_PE_THREE", "ONLINE_APPLY_PRICE", "MAIN_BUSINESS",
            "PAGE_PREDICT_PRICE1", "PAGE_PREDICT_PRICE2", "PAGE_PREDICT_PRICE3", "PAGE_PREDICT_PE1",
            "PAGE_PREDICT_PE2", "PAGE_PREDICT_PE3", "SELECT_LISTING_DATE", "IS_BEIJING", "INDUSTRY_PE_RATIO",
            "INDUSTRY_PE", "IS_REGISTRATION",
        ]
        params = {
            "sortColumns": "APPLY_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "reportName": "RPTA_APP_IPOAPPLY",
            "columns": ",".join(columns),
            "quoteColumns": "f2~01~SECURITY_CODE~NEWEST_PRICE",
            "quoteType": "0",
            "filter": self.params_filter(),
        }
        params.update(self.params_update())
        return self.base_param(update=params)



"""
https://np-listapi.eastmoney.com/comm/web/media/news?
biz=media_news&client=web&needInteractData=1&sortEnd=0&page_size=20
&fields=code,title,image,summary,showTime,mediaName,titleColor,uniqueUrl,realSort,tagCodes,seoKeys
&req_trace=1727577461402&callback=jQuery1830807191840992131_1727577461312&_=1727577461402
"""



import json
from typing import Any, List, Dict, Tuple, Callable, Optional
from wisecon.types import BaseMapping, BaseRequestData, ResponseData, Metadata


__all__ = [
    "FundCompanyMapping",
    "FundCompany",
]


class FundCompanyMapping(BaseMapping):
    """"""
    columns: Dict = {
        "company_code": "公司代码",
        "company_name": "公司名称",
    }


class FundCompany(BaseRequestData):
    """ Fund Company """
    def __init__(
            self,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """"""
        self.mapping = FundCompanyMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

        self.request_set(
            response_type="text",
            description="基金公司代码",
        )

    def base_url(self) -> str:
        """"""
        base_url = "http://fund.eastmoney.com/js/jjjz_gs.js"
        return base_url

    def clean_content(
            self,
            content: Optional[str],
    ) -> List[Dict]:
        """"""
        columns = list(self.mapping.columns.keys())
        content = content[content.find("["): content.rfind("]") + 1]
        data = json.loads(content)
        data = [dict(zip(columns, item)) for item in data]
        return data



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "CustomsMapping",
    "Customs",
]


class CustomsMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "EXIT_BASE": "当月出口额（亿美金）",
        "IMPORT_BASE": "当月进口额（亿美金）",
        "EXIT_BASE_SAME": "当月出口额（同比）",
        "IMPORT_BASE_SAME": "当月进口额（同比）",
        "EXIT_BASE_SEQUENTIAL": "当月出口额（环比）",
        "IMPORT_BASE_SEQUENTIAL": "当月进口额（环比）",
        "EXIT_ACCUMULATE": "累计出口额（亿美金）",
        "IMPORT_ACCUMULATE": "累计进口额（亿美金）",
        "EXIT_ACCUMULATE_SAME": "累计出口额（同比）",
        "IMPORT_ACCUMULATE_SAME": "累计进口额（同比）",
    }


class Customs(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = CustomsMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 海关进出口增减情况一览表",
        )

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "EXIT_BASE", "IMPORT_BASE", "EXIT_BASE_SAME", "IMPORT_BASE_SAME",
            "EXIT_BASE_SEQUENTIAL", "IMPORT_BASE_SEQUENTIAL", "EXIT_ACCUMULATE", "IMPORT_ACCUMULATE",
            "EXIT_ACCUMULATE_SAME", "IMPORT_ACCUMULATE_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_CUSTOMS",
        }
        return params



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "StockOpenMapping",
    "StockOpen",
]


class StockOpenMapping(BaseMapping):
    """"""
    columns: Dict = {
        "STATISTICS_DATE": "数据日期",
        "ADD_INVESTOR": "新增投资者(万户)",
        "ADD_INVESTOR_QOQ": "新增投资者(环比)",
        "ADD_INVESTOR_YOY": "新增投资者(同比)",
        "END_INVESTOR": "期末投资者(万户)",
        "END_INVESTOR_A": "期末投资者(A股)",
        "END_INVESTOR_B": "期末投资者(B股)",
        "CLOSE_PRICE": "上证指数(收盘)",
        "CHANGE_RATE": "上证指数(涨跌幅)",
        "TOTAL_MARKET_CAP": "沪深总市值",
        "AVERAGE_MARKET_CAP": "沪深户均市值",
        "STATISTICS_DATE_NY": "月份"
    }


class StockOpen(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = StockOpenMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 股票账户统计详细数据",
        )

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "STATISTICS_DATE", "ADD_INVESTOR", "ADD_INVESTOR_QOQ", "ADD_INVESTOR_YOY", "END_INVESTOR",
            "END_INVESTOR_A", "END_INVESTOR_B", "CLOSE_PRICE", "CHANGE_RATE", "TOTAL_MARKET_CAP",
            "AVERAGE_MARKET_CAP", "STATISTICS_DATE_NY",
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "STATISTICS_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_STOCK_OPEN_DATA",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "StockRankStatisticMapping",
    "StockRankStatistic",
]


class StockRankStatisticMapping(BaseMapping):
    """字段映射 个股上榜统计"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "LATEST_TDATE": "最新交易日期",
        "SECURITY_NAME_ABBR": "证券简称",
        "IPCT1M": "1个月涨幅",
        "IPCT3M": "3个月涨幅",
        "IPCT6M": "6个月涨幅",
        "IPCT1Y": "1年涨幅",
        "CHANGE_RATE": "变动率",
        "CLOSE_PRICE": "收盘价格",
        "PERIOD": "统计周期",
        "BILLBOARD_DEAL_AMT": "排行榜成交金额",
        "BILLBOARD_NET_BUY": "排行榜净买入",
        "ORG_TIMES": "机构交易次数",
        "ORG_DEAL_AMT": "机构成交金额",
        "ORG_NET_BUY": "机构净买入",
        "BILLBOARD_TIMES": "排行榜交易次数",
        "BILLBOARD_BUY_AMT": "排行榜买入金额",
        "BILLBOARD_SELL_AMT": "排行榜卖出金额",
        "ORG_BUY_AMT": "机构买入金额",
        "ORG_SELL_AMT": "机构卖出金额",
        "ORG_BUY_TIMES": "机构买入次数",
        "ORG_SELL_TIMES": "机构卖出次数",
        "STATISTICS_CYCLE": "统计周期",
        "SECURITY_TYPE_CODE": "证券类型代码"
    }


class StockRankStatistic(APIDataV1RequestData):
    """查询 个股上榜统计"""

    def __init__(
            self,
            statistics_cycle: Literal["1m", "3m", "6m", "12m"] = "1m",
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = StockRankStatistic(statistics_cycle="1m").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            statistics_cycle: 统计周期, 可选值: 1m, 3m, 6m, 12m, 默认值: 1m
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.statistics_cycle = statistics_cycle
        self.size = size
        self.mapping = StockRankStatisticMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="个股上榜统计",)
        self.conditions = []

    def params_statistics_cycle(self):
        """"""
        cycle_mapping = {"1m": "01", "3m": "02", "6m": "03", "12m": "04"}
        self.conditions.append(f'(STATISTICS_CYCLE="{cycle_mapping.get(self.statistics_cycle, "01")}")')

    def params_filter(self) -> str:
        """"""
        self.params_statistics_cycle()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "BILLBOARD_TIMES,LATEST_TDATE,SECURITY_CODE",
            "sortTypes": "-1,-1,1",
            "reportName": "RPT_BILLBOARD_TRADEALLNEW",
        }
        return self.base_param(update=params)



import types
import inspect
from enum import EnumMeta
from typing import *
from types import GenericAlias
from typing import List, Dict
from wisecon.types.agent import ToolParameters, ToolItem, ToolFunction


__all__ = [
    "enum_metadata",
    "register_tool",
]


def enum_metadata(typ: EnumMeta) -> Dict:
    """"""
    enum_typ_lst = list(typ)
    enum_properties = dict()
    info = dict()
    for item in enum_typ_lst:
        enum_item = eval(item.value)
        enum_typ, (enum_desc, enum_name) = enum_item.__origin__, enum_item.__metadata__
        enum_properties.update({"type": enum_typ.__name__,})
        info.update({"value": enum_name, "description": enum_desc})
    enum_properties.update({"enum": info})
    return enum_properties


def register_tool(hooks: Dict, descriptions: List):
    """"""
    def decorator_func(func):
        tool_name = func.__name__
        if isinstance(func, types.FunctionType):
            tool_description = inspect.getdoc(func).strip()
            python_params = inspect.signature(func).parameters
        elif isinstance(func, type):
            tool_description = inspect.getdoc(func.__init__).strip()
            python_params = dict(inspect.signature(func.__init__).parameters)
            _ = python_params.pop("self")
        else:
            raise TypeError(f"Only function or class can be decorated, but got {type(func)}")
        tool_params = []
        required_lst = []
        properties = dict()
        for name, param in python_params.items():
            annotation = param.annotation
            if annotation is inspect.Parameter.empty:
                raise TypeError(f"Parameter `{name}` missing type annotation")
            if get_origin(annotation) != Annotated:
                raise TypeError(f"Annotation type for `{name}` must be typing.Annotated")

            typ, (description, required) = annotation.__origin__, annotation.__metadata__

            if isinstance(typ, EnumMeta):
                enum_properties = enum_metadata(typ)
            else:
                enum_properties = None

            typ: str = str(typ) if isinstance(typ, GenericAlias) else typ.__name__
            if not isinstance(description, str):
                raise TypeError(f"Description for `{name}` must be a string")
            if not isinstance(required, bool):
                raise TypeError(f"Required for `{name}` must be a bool")

            if required:
                required_lst.append(name)

            tool_params.append({
                "name": name,
                "description": description,
                "type": typ,
                "required": required
            })

            properties.update({name: dict()})
            properties.get(name).update({
                "description": description,
                "type": typ,
            })
            if enum_properties:
                properties.get(name).update({
                    "enum": enum_properties.get("enum"),
                    "type": enum_properties.get("type"),
                })

        tool_def = {
            "name": tool_name,
            "description": tool_description,
            "parameters": tool_params
        }
        hooks[tool_name] = func

        tool_parameters = ToolParameters(
            properties=properties,
            required=required_lst,
        )
        tool_item = ToolItem(
            function=ToolFunction(
                name=tool_def.get("name"),
                description=tool_def.get("description"),
                parameters=tool_parameters
            )
        )
        descriptions.append(tool_item.model_dump())

        def wrapper(*args, **kwargs):
            """"""
            return func
        return wrapper
    return decorator_func



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "StockDividend",
    "StockDividendMapping",
]


class StockDividendMapping(BaseMapping):
    """字段映射 上市公司分红数据"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "SECURITY_INNER_CODE": "内部代码",
        "ORG_CODE": "机构代码",
        "SECURITY_CODE": "证券代码",
        "BONUS_IT_RATIO": "股息税后比例",
        "BONUS_RATIO": "股息比例",
        "IT_RATIO": "分配比例",
        "PRETAX_BONUS_RMB": "税前股息（人民币）",
        "PLAN_NOTICE_DATE": "计划通知日期",
        "EQUITY_RECORD_DATE": "股权登记日期",
        "EX_DIVIDEND_DATE": "除息日期",
        "REPORT_DATE": "报告日期",
        "ASSIGN_PROGRESS": "方案进度",
        "IMPL_PLAN_PROFILE": "现金分红比例",
        "NOTICE_DATE": "公告日期",
        "MARKET_TYPE": "市场类型",
        "EX_DIVIDEND_DAYS": "除息天数",
        "IS_KCB": "是否科创板",
        "BASIC_EPS": "每股收益(元)",
        "BVPS": "每股净资产(元)",
        "PER_CAPITAL_RESERVE": "每股公积金(元)",
        "PER_UNASSIGN_PROFIT": "每股未分配利润(元)",
        "PNP_YOY_RATIO": "净利润同比增长率(%)",
        "TOTAL_SHARES": "总股本(元)",
        "PUBLISH_DATE": "发布日期",
        "DIVIDENT_RATIO": "股息率",
        "D10_CLOSE_ADJCHRATE": "10日收盘价调整变化率",
        "BD10_CLOSE_ADJCHRATE": "前10日收盘价调整变化率",
        "D30_CLOSE_ADJCHRATE": "30日收盘价调整变化率"
    }


class StockDividend(APIDataV1RequestData):
    """查询 上市公司分红数据"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import StockDividend

            data = StockDividend(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = StockDividendMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司分红数据")

    def params_filter(self) -> str:
        """"""
        conditions = []
        if self.start_date:
            conditions.append(f"(REPORT_DATE>='{self.start_date}')")
        if self.end_date:
            conditions.append(f"(REPORT_DATE<='{self.end_date}')")
        if self.date:
            conditions.append(f"(REPORT_DATE='{self.date}')")
        if self.security_code:
            conditions.append(f'(SECURITY_CODE="{self.security_code}")')
        return "".join(conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "PLAN_NOTICE_DATE",
            "sortTypes": -1,
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_SHAREBONUS_DET",
        }
        return self.base_param(params)



from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "StockRankDetailMapping",
    "StockRankDetail",
]


class StockRankDetailMapping(BaseMapping):
    """字段映射 个股上榜统计"""
    columns: Dict = {
        "SECURITY_CODE": "证券代码",
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "TRADE_DATE": "交易日期",
        "EXPLAIN": "说明",
        "CLOSE_PRICE": "收盘价格",
        "CHANGE_RATE": "变动率",
        "BILLBOARD_NET_AMT": "排行榜净金额",
        "BILLBOARD_BUY_AMT": "排行榜买入金额",
        "BILLBOARD_SELL_AMT": "排行榜卖出金额",
        "BILLBOARD_DEAL_AMT": "排行榜成交金额",
        "ACCUM_AMOUNT": "累计金额",
        "DEAL_NET_RATIO": "成交净比率",
        "DEAL_AMOUNT_RATIO": "成交金额比率",
        "TURNOVERRATE": "换手率",
        "FREE_MARKET_CAP": "自由流通市值",
        "EXPLANATION": "解释",
        "D1_CLOSE_ADJCHRATE": "D1日收盘调整幅度",
        "D2_CLOSE_ADJCHRATE": "D2日收盘调整幅度",
        "D5_CLOSE_ADJCHRATE": "D5日收盘调整幅度",
        "D10_CLOSE_ADJCHRATE": "D10日收盘调整幅度",
        "SECURITY_TYPE_CODE": "证券类型代码"
    }


class StockRankDetail(APIDataV1RequestData):
    """查询 个股上榜统计"""

    def __init__(
            self,
            market: Optional[Literal["沪市A股", "科创板", "深市A股", "创业板", "京市A股"]] = None,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = StockRankDetail().load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场类型, 可选值: 沪市A股, 科创板, 深市A股, 创业板, 京市A股, 默认值: None
            start_date: 开始日期, 格式: yyyy-MM-dd, 默认值: None
            end_date: 结束日期, 格式: yyyy-MM-dd, 默认值: None
            date: 日期, 格式: yyyy-MM-dd, 默认值: None
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.market = market
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.size = size
        self.mapping = StockRankDetailMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="龙虎榜详情",)
        self.conditions = []
        self.validate_date_format(date=[date, start_date, end_date])

    def params_market(self):
        """"""
        market_mapping = {
            "沪市A股": '(TRADE_MARKET_CODE in ("069001001001","069001001006","069001001003"))',
            "科创板": '(TRADE_MARKET_CODE="069001001006")',
            "深市A股": '(TRADE_MARKET_CODE in ("069001002001","069001002002","069001002005"))',
            "创业板": '(TRADE_MARKET_CODE="069001002002")',
            "京市A股": '(TRADE_MARKET_CODE="069001017")',
        }
        if self.market:
            self.conditions.append(market_mapping.get(self.market))

    def params_filter(self) -> str:
        """"""
        self.params_market()
        self.filter_report_date(date_name="TRADE_DATE")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "SECURITY_CODE", "SECUCODE", "SECURITY_NAME_ABBR", "TRADE_DATE", "EXPLAIN",
            "CLOSE_PRICE", "CHANGE_RATE", "BILLBOARD_NET_AMT", "BILLBOARD_BUY_AMT",
            "BILLBOARD_SELL_AMT", "BILLBOARD_DEAL_AMT", "ACCUM_AMOUNT", "DEAL_NET_RATIO",
            "DEAL_AMOUNT_RATIO", "TURNOVERRATE", "FREE_MARKET_CAP", "EXPLANATION", "D1_CLOSE_ADJCHRATE",
            "D2_CLOSE_ADJCHRATE", "D5_CLOSE_ADJCHRATE", "D10_CLOSE_ADJCHRATE", "SECURITY_TYPE_CODE"
        ]

        params = {
            "columns": ",".join(columns),
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "SECURITY_CODE,TRADE_DATE",
            "sortTypes": "1,-1",
            "reportName": "RPT_DAILYBILLBOARD_DETAILSNEW",
        }
        return self.base_param(update=params)
